// ============================================================================
// $Id: FunctorScript.java,v 1.2 2009/01/19 03:37:21 davidahall Exp $
// Copyright (c) 2007  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not
// use this file except in compliance with the License.
//
// From time to time, the license steward (initially Sun Microsystems,
// Inc.), and may publish revised and/or new versions of this License.
// You may not use, distribute, or otherwise make this file available
// under subsequent versions of the License.
//
// You should have received a copy of the the License along with this
// file: if not, a copy of the License is available at
//
// http://www.sun.com/cddl/cddl.html
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// ============================================================================

package net.sf.jga.scripting;

import javax.script.Bindings;
import javax.script.CompiledScript;
import javax.script.ScriptContext;
import javax.script.ScriptEngine;
import javax.script.ScriptException;

import net.sf.jga.fn.UnaryFunctor;

/**
 * FunctorScript.java
 *
 * <p>
 * Copyright &copy; 2007  David Hall
 * @author <a href="mailto:davidahall@users.sf.net">David Hall</a>
 */

public class FunctorScript extends CompiledScript {
    
    private ScriptEngine engine;
    private UnaryFunctor<ScriptContext,?> functor;
    
    public FunctorScript (ScriptEngine engine, UnaryFunctor<ScriptContext,?> fn){
        this.engine = engine;
        this.functor = fn;
    }

    // ------------------------
    // CompiledScript interface
    // ------------------------
    
    public Object eval(ScriptContext context) throws ScriptException {
        return functor.fn(context);
    }

    public Object eval(Bindings bindings) throws ScriptException {
        Bindings oldBindings = engine.getBindings(ScriptContext.ENGINE_SCOPE);
        engine.setBindings(bindings, ScriptContext.ENGINE_SCOPE);
        try {
            return eval(engine.getContext());
        }
        finally {
            engine.setBindings(oldBindings, ScriptContext.ENGINE_SCOPE);
        }
    }

    public Object eval() throws ScriptException {
        return functor.fn(engine.getContext());
    }

    public ScriptEngine getEngine() {
        return engine;
    }
}
