// ============================================================================
// $Id: JFXGScriptEngine.java,v 1.2 2009/01/19 03:37:21 davidahall Exp $
// Copyright (c) 2006  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.scripting;

import java.io.Reader;
import java.io.StringReader;
import javax.script.AbstractScriptEngine;
import javax.script.Bindings;
import javax.script.Compilable;
import javax.script.CompiledScript;
import javax.script.ScriptContext;
import javax.script.ScriptEngineFactory;
import javax.script.ScriptException;
import javax.script.SimpleBindings;
import net.sf.jga.fn.UnaryFunctor;
import net.sf.jga.parser.ParseException;

public class JFXGScriptEngine extends AbstractScriptEngine implements Compilable {

    private ScriptParser parser = new ScriptParser();

    public final Object eval(final String string, final ScriptContext scriptContext)
        throws ScriptException
    {
        CompiledScript script = compile(string, scriptContext);
        return script.eval(scriptContext);
    }

    public final Object eval(final Reader reader, final ScriptContext scriptContext)
        throws ScriptException
    {
        CompiledScript script = compile(reader, scriptContext);
        return script.eval(scriptContext);
    }

    public final ScriptEngineFactory getFactory() {
        return new JFXGScriptEngineFactory();
    }

    public Bindings createBindings() {
        return new SimpleBindings();
    }

    // -------------------------
    // Compilable implementation
    // -------------------------

    public final CompiledScript compile(final String string) throws ScriptException {
        return compile(string, getContext());
    }

    public final CompiledScript compile(final Reader reader) throws ScriptException {
        return compile(reader, getContext());
    }

    // -------------------------
    // implementation details
    // -------------------------

    public final CompiledScript compile(final String string, ScriptContext context)
        throws ScriptException
    {
        return compile(new StringReader(string), context);
    }

    public final CompiledScript compile(final Reader reader, ScriptContext context)
        throws ScriptException
    {
        try {
            parser.setScriptContext(context);
            parser.ReInit(reader);
            UnaryFunctor<ScriptContext,?> fn = parser.parseUnary(ScriptContext.class);
            return new FunctorScript(this, fn);
        }
        catch (ParseException x) {
            throw new ScriptException(x);
        }
    }
}
