// ============================================================================
// $Id: JFXGScriptEngineFactory.java,v 1.2 2009/01/19 03:37:21 davidahall Exp $
// Copyright (c) 2006  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.scripting;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineFactory;

public class JFXGScriptEngineFactory implements ScriptEngineFactory {

    public final String getEngineVersion() {
        // TODO: Read this value from the jfxg-engine.jar manifest
        return "0.1";
    }

    public final String getEngineName() {
        return "jfxg";
    }

    public final String getLanguageName() {
        return "jfxg";
    }

    public final String getLanguageVersion() {
        // TODO: Read this value from the jga.jar manifest
        return "0.8.1";
    }

    public final ScriptEngine getScriptEngine() {
        return new JFXGScriptEngine();
    }

    public final List<String> getNames() {
        String[] names = { "jfxg" };
        return Arrays.asList(names);
    }

    public final List<String> getMimeTypes() {
        return new ArrayList<String>();
    }

    public final List<String> getExtensions() {
        String[] ext = { "jfxg" };
        return Arrays.asList(ext);
    }
    
    public final Object getParameter(final String key) {
        if (key.equals(ScriptEngine.ENGINE))
            return getEngineName();

        if (key.equals(ScriptEngine.ENGINE_VERSION))
            return getEngineVersion();

        if (key.equals(ScriptEngine.NAME))
            return getNames().get(0);

        if (key.equals(ScriptEngine.LANGUAGE))
            return getLanguageName();

        if (key.equals(ScriptEngine.LANGUAGE_VERSION))
            return getLanguageVersion();

        return null;
    }
        
    public final String getMethodCallSyntax(final String obj, final String m, final String... args) {
        StringBuffer buf = new StringBuffer();
        buf.append(obj).append('.').append(m).append("(");

        String fs = "";
        for(String arg : args) {
            buf.append(fs).append(arg);
            fs = ",";
        }

        buf.append(");");
        return buf.toString();
    }

    public final String getOutputStatement(final String toDisplay) {
        return "System.out.println(\""+toDisplay+"\");";
    }

    public final String getProgram(final String... statements) {
        StringBuffer buf = new StringBuffer();
        for(String stmt : statements) {
            buf.append(stmt);
        }

        return buf.toString();
    }
}
