// ============================================================================
// $Id: Flatten.java,v 1.3 2009/09/12 20:42:07 davidahall Exp $
// Copyright (c) 2006  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.algorithms;

import java.util.Collection;
import java.util.Iterator;

import net.sf.jga.util.CollectionUtils;

/**
 * Algorithms that two-dimensional input, ie, arrays of arrays, collections of collections, etc.
 * <p>
 * Adapted from a suggestion by Be&ntilde;at Gurmendi.
 * <p>
 * Copyright &copy; 2008  David A. Hall
 */

public class Flatten {

    // ============
    // Arrays
    // ============
    
    /**
     * Returns all elements of the 2d array
     */
    static public <T> Iterable<T> flatten(T[][] ts) {
        return Append.append(ts);
    }
 
    // ============
    // Iterables
    // ============
    
    /**
     * Returns an Iterable that 'contains' all of the elements of the input(s)
     */
    static public <T> Iterable<T> flatten(Iterable<? extends Iterable<? extends T>> input) {
        return new Append.AppendIterable<T>(input);
    }
    
    // ============
    // Iterators
    // ============
    
    /**
     * Returns an Iterator that will return elements from all of the iterators
     */
    static public <T> Iterator<T> flatten(Iterator<? extends Iterator<? extends T>> input) {
        return new Append.AppendIterator<T>(input);
    }

    // ============
    // IterableCopy
    // ============
    
    /**
     * Appends the values of all of the intpu to the output collection
     * @return the output collection
     */
    static public <T, TCollection extends Collection<? super T>> TCollection
    flatten(Iterable<? extends Iterable<? extends T>> input, TCollection cout) {
        return CollectionUtils.append(cout, flatten(input));
    }
}
