// ============================================================================
// $Id: Generator.java,v 1.9 2009/08/14 12:12:10 davidahall Exp $
// Copyright (c) 2004-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn;

import java.io.Serializable;

/**
 * Functor that takes no arguments and produces a single result.  The result is
 * not guaranteed to be the same each time the functor is called.
 * <p>
 * Copyright &copy; 2004-2005  David A. Hall
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

abstract public class Generator<R> extends Functor<R> implements Serializable, Visitable {
    /**
     * Generate one instance.
     */
    abstract public R fn();

    // -----------------
    // Functor interface
    // -----------------
    
    /**
     * Executes the function and returns the result.
     */
    public R eval(Object... args) {
        return fn();
    }
    
    /**
     * Binds the given values to this functor, starting from the first argument.
     * @param arg0
     * @param args
     * @return
     */
    @Override
    public Functor<R> bind0th(Object arg0, Object... args) {
        return this; // args bound out of bounds would be ignored anyway
    }
    
    /**
     * Binds the given values to this functor, starting from the given index.
     * @param idx
     * @param arg0
     * @param args
     * @return
     */
    @Override
    public Functor<R> bindNth(int idx, Object arg0, Object... args) {
        return this; // args bound out of bounds would be ignored anyway
    }
}
