// ============================================================================
// $Id: Visitable.java,v 1.5 2009/01/19 03:37:21 davidahall Exp $
// Copyright (c) 2002-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn;

/**
 * Defines an interface for classes that may be interpreted by a Visitor.
 * <p>
 * jga uses the <a href="http://www.objectmentor.com/resources/articles/acv.pdf">AcyclicVisitor</a>
 * pattern to provide a structure for implementing Visitor.  Within jga, each
 * class that implements Visitable provides a nested interface called Visitor
 * that defines a <code>visit</code> method for the class.  This is typically
 * implemented with the following boilerplate:<br>
 * <pre>
 * public class Foo implements Visitable
 *    public void accept(Visitor v) {
 *          v.visit(this);
 *    }
 *
 *    public interface Visitor extends net.sf.jga.fn.Visitor {
 *        public void visit(Foo host); 
 *    }
 * </pre>
 * <p>
 * Copyright &copy; 2002-2005  David A. Hall
 *
 * @author <a href="mailto:davidahall@users.sourceforge.net">David A. Hall</a>
 **/

public interface Visitable {
    /**
     * Determine if the visitor is appropriate (typically by testing against
     * a specific interface) and if so, call pass the implementing object to
     * the visitor's visit method.
     */
    public void accept(Visitor visitor);
}
