// ============================================================================
// $Id: AndGenerator.java,v 1.9 2007/02/07 12:20:48 davidahall Exp $
// Copyright (c) 2002-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.adaptor;

import net.sf.jga.fn.Generator;

/**
 * Generator that performs a shortcircuit and operation using a given pair
 * of Boolean Generators.  When the first generator returns true, the second
 * is not evaluated.
 **/

public class AndGenerator extends Generator<Boolean> {

    static final long serialVersionUID = 6260506966868219143L;
    
    // The two generators that are tested
    private Generator<Boolean> _first;
    private Generator<Boolean> _second;
    
    /**
     * Builds a AndGenerator functor, given the two functors that may be executed.
     * @throws IllegalArgumentException if any of the functors is missing
     */
    public AndGenerator(Generator<Boolean> first, Generator<Boolean> second) {
        if (first == null || second == null) {
            throw new IllegalArgumentException("Two functors are required");
        }
        
        _first = first;
        _second = second;
    }


    public Generator<Boolean> getFirstFunctor() { return _first; }

    public Generator<Boolean> getSecondFunctor() { return _second; }
    
    
    public Boolean fn() {
        return _first.fn() && _second.fn();
    }

    /**
     * Calls the Visitor's <code>visit(AndGenerator)</code> method, if it
     * implements the nested Visitor interface.
     */
    public void accept(net.sf.jga.fn.Visitor v) {
        if (v instanceof AndGenerator.Visitor)
            ((AndGenerator.Visitor)v).visit(this);
        else
            v.visit(this);
    }

    // Object overrides

    public String toString() {
        return _first +" && " +_second;
    }
    
    // AcyclicVisitor
    
    /**
     * Interface for classes that may interpret an <b>AndGenerator</b> predicate.
     */
    public interface Visitor extends net.sf.jga.fn.Visitor {
        public void visit(AndGenerator host);
    }
}
