// ============================================================================
// $Id: ApplyUnary.java,v 1.10 2009/01/19 03:37:21 davidahall Exp $
// Copyright (c) 2004-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.adaptor;

import net.sf.jga.fn.UnaryFunctor;
import net.sf.jga.util.ArrayUtils;

/**
 * Produces an array containing the results of  passing an input argument to a given set of unary
 * functors.
 * <p>
 * Copyright &copy; 2004-2005  David A. Hall
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

public class ApplyUnary<T> extends UnaryFunctor<T,Object[]> {

    static final long serialVersionUID = -7934367561074978884L;

    private UnaryFunctor<T,?>[] _functors;
    
    public ApplyUnary (UnaryFunctor<T,?>... functors){
        _functors = functors;
    }

    public UnaryFunctor<T,?>[] getFunctors() {
        return _functors;
    }                                                 

    public Object[] fn(T arg) {
        Object[] result = new Object[_functors.length];
        for (int i = 0; i < _functors.length; ++i) {
            result[i] = _functors[i].fn(arg);
        }

        return result;
    }

    /**
     * Calls the Visitor's <code>visit(ApplyUnary)</code> method, if it
     * implements the nested Visitor interface.
     */
    public void accept(net.sf.jga.fn.Visitor v) {
        if (v instanceof ApplyUnary.Visitor)
            ((ApplyUnary.Visitor)v).visit(this);
        else
            v.visit(this);
    }

    // Object overrides

    public String toString() {
        return ArrayUtils.toString(_functors);
    }
    
    // AcyclicVisitor
    
    /**
     * Interface for classes that may interpret an <b>ApplyUnary</b> predicate.
     */
    public interface Visitor extends net.sf.jga.fn.Visitor {
        public void visit(ApplyUnary<?> host);
    }
}
