// ============================================================================
// $Id: Bind2nd.java,v 1.13 2009/01/19 03:37:21 davidahall Exp $
// Copyright (c) 2003-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.adaptor;

import net.sf.jga.fn.BinaryFunctor;
import net.sf.jga.fn.UnaryFunctor;

/**
 * UnaryFunctor that wraps a given BinaryFunctor, passing a constant value
 * as the second argument of the child functor.  The runtime argument is passed
 * as the first argument of the child functor.
 * <p>
 * Copyright &copy; 2003-2005  David A. Hall
 *
 * @author <a href="mailto:davidahall@users.sourceforge.net">David A. Hall</a>
 **/

public class Bind2nd<T1, T2, R> extends UnaryFunctor<T1, R> {

    static final long serialVersionUID = -5977756270123944716L;
    
    // The functor whose second argument is bound
    private BinaryFunctor<T1, T2, R> _f;
    
    // The value bound to the second arg
    private T2 _c;

    /**
     * Builds a Bind2nd Functor with the given contant and child functor.
     * @throws IllegalArgumentException if the functor is null.
     */
    public Bind2nd(T2 constant, BinaryFunctor<T1, T2, R> fn) {
        if (fn == null) {
            throw new IllegalArgumentException("Must supply a function");
        }
        _f = fn;
        _c = constant;
    }


    /**
     * Returns the child functor for this functor
     * @return the child functor for this functor
     */
    public BinaryFunctor<T1, T2, R> getFunctor() { return _f; }
    
    /**
     * Returns the constant value
     * @return the constant value
     */
    public T2 getConstant() { return _c; }

    // UnaryFunctor interface
    
    /**
     * Given one argument, passes the argument and the constant value to the
     * child functor and returns the result.
     *
     * @return f(x,c)
     */
    public R fn(T1 x) {
        return _f.fn(x, _c);
    }

    /**
     * Calls the Visitor's <code>visit(Bind2nd)</code> method, if it
     * implements the nested Visitor interface.
     */
    public void accept(net.sf.jga.fn.Visitor v) {
        if (v instanceof Bind2nd.Visitor)
            ((Bind2nd.Visitor)v).visit(this);
        else
            v.visit(this);
    }
    
    // Object overrides

    public String toString() {
        return _f+".bind2nd("+_c+")";
    }
    
    // AcyclicVisitor
    
    /**
     * Interface for classes that may interpret an <b>Bind2nd</b> functor.
     */
    public interface Visitor extends net.sf.jga.fn.Visitor {
        public void visit(Bind2nd<?,?,?> bf);
    }
}
