// ============================================================================
// $Id: Compound.java,v 1.1 2009/06/27 23:44:51 davidahall Exp $
// Copyright (c) 2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.adaptor;

import net.sf.jga.fn.Functor;

/**
 * Executes two Functors, returning the results of the second.
 * <p>
 * Copyright &copy; 2005  David A. Hall
 */
public class Compound<R> extends Functor<R> {
    private static final long serialVersionUID = 4143922554251107555L;

    // the first functor.  Results of this functor are dropped
    private Functor<?> _fn1;

    // the second functor.  Results of this functor are returned.
    private Functor<R> _fn2;
    
    public Compound(Functor<?> fn1, Functor<R> fn2) {
        if (fn1 == null || fn2 == null)
            throw new IllegalArgumentException("Two functors are required");

        _fn1 = fn1;
        _fn2 = fn2;
    }

    
    /**
     * @return the first of the two nested functors
     */
    public Functor<?> getFirstFunctor() { return _fn1; }

    
    /**
     * @return the second of the two nested functors
     */
    public Functor<R> getSecondFunctor() { return _fn2; }

    
    @Override
    public R eval(Object... args) {
        _fn1.eval(args);
        return _fn2.eval(args); 
    }

    /**
     * Calls the Visitor's <code>visit(Compound)</code> method, if it
     * implements the nested Visitor interface.
     */
    public void accept(net.sf.jga.fn.Visitor v) {
        if (v instanceof Compound.Visitor)
            ((Compound.Visitor)v).visit(this);
        else
            v.visit(this);
    }
    
    // Object overrides

    public String toString() {
        return _fn1 +"," +_fn2;
    }
    
     // AcyclicVisitor
     
    /**
     * Interface for classes that may interpret a <b>Constant</b> binaryFunctor.
     */
    public interface Visitor extends net.sf.jga.fn.Visitor {
        public void visit(Compound<?> host);
    }
}
