// ============================================================================
// $Id: CompoundBinary.java,v 1.6 2009/06/27 23:43:38 davidahall Exp $
// Copyright (c) 2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.adaptor;

import net.sf.jga.fn.BinaryFunctor;

/**
 * Executes two BinaryFunctors, returning the results of the second.
 * <p>
 * Copyright &copy; 2005  David A. Hall
 */

public class CompoundBinary<T1,T2,R> extends BinaryFunctor<T1,T2,R> {
    
    static final long serialVersionUID = -6716796579409524752L;
    
    // the first functor.  Results of this functor are dropped
    private BinaryFunctor<T1,T2,?> _fn1;

    // the second functor.  Results of this functor are returned.
    private BinaryFunctor<T1,T2,R> _fn2;
    
    public CompoundBinary (BinaryFunctor<T1,T2,?> fn1, BinaryFunctor<T1,T2,R> fn2) {
        if (fn1 == null || fn2 == null)
            throw new IllegalArgumentException("Two functors are required");

        _fn1 = fn1;
        _fn2 = fn2;
    }

    
    /**
     * @return the first of the two nested functors
     */
    public BinaryFunctor<T1,T2,?> getFirstFunctor() { return _fn1; }

    
    /**
     * @return the second of the two nested functors
     */
    public BinaryFunctor<T1,T2,R> getSecondFunctor() { return _fn2; }

    
    /**
     * Executes both functors, returning the results of the second.
     */
    public R fn(T1 arg1, T2 arg2) {
        _fn1.fn(arg1, arg2);
        return _fn2.fn(arg1, arg2);
    }

    
    /**
     * Calls the Visitor's <code>visit(CompoundBinary)</code> method, if it
     * implements the nested Visitor interface.
     */
    public void accept(net.sf.jga.fn.Visitor v) {
        if (v instanceof CompoundBinary.Visitor)
            ((CompoundBinary.Visitor)v).visit(this);
        else
            v.visit(this);
    }
    
    // Object overrides

    public String toString() {
        return _fn1 +"," +_fn2;
    }
    
     // AcyclicVisitor
     
    /**
     * Interface for classes that may interpret a <b>Constant</b> binaryFunctor.
     */
    public interface Visitor extends net.sf.jga.fn.Visitor {
        public void visit(CompoundBinary<?,?,?> host);
    }
}
