// ============================================================================
// $Id: CompoundUnary.java,v 1.5 2009/01/19 03:37:21 davidahall Exp $
// Copyright (c) 2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.adaptor;

import net.sf.jga.fn.UnaryFunctor;

/**
 * Executes two UnaryFunctors, returning the results of the second.
 * <p>
 * Copyright &copy; 2005  David A. Hall
 */

public class CompoundUnary<T,R> extends UnaryFunctor<T,R> {
    
    static final long serialVersionUID = 3041907427002719706L;

    // the first functor.  Results of this functor are dropped
    private UnaryFunctor<T,?> _fn1;

    // the second functor.  Results of this functor are returned.
    private UnaryFunctor<T,R> _fn2;
    
    public CompoundUnary (UnaryFunctor<T,?> fn1, UnaryFunctor<T,R> fn2) {
        if (fn1 == null || fn2 == null)
            throw new IllegalArgumentException("Two functors are required");

        _fn1 = fn1;
        _fn2 = fn2;
    }

    
    /**
     * @return the first of the two nested functors
     */
    public UnaryFunctor<T,?> getFirstFunctor() { return _fn1; }

    
    /**
     * @return the second of the two nested functors
     */
    public UnaryFunctor<T,R> getSecondFunctor() { return _fn2; }

    
    /**
     * Executes both functors, returning the results of the second.
     */
    public R fn(T arg) {
        _fn1.fn(arg);
        return _fn2.fn(arg);
    }

    
    /**
     * Calls the Visitor's <code>visit(Constant)</code> method, if it
     * implements the nested Visitor interface.
     */
    public void accept(net.sf.jga.fn.Visitor v) {
        if (v instanceof CompoundUnary.Visitor)
            ((CompoundUnary.Visitor)v).visit(this);
        else
            v.visit(this);
    }
    
    // Object overrides

    public String toString() {
        return _fn1 +"," +_fn2;
    }
    
     // AcyclicVisitor
     
    /**
     * Interface for classes that may interpret a <b>Constant</b> unaryFunctor.
     */
    public interface Visitor extends net.sf.jga.fn.Visitor {
        public void visit(CompoundUnary<?,?> host);
    }
}
