// ============================================================================
// $Id: Generate2nd.java,v 1.9 2009/01/19 03:37:21 davidahall Exp $
// Copyright (c) 2004-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.adaptor;

import net.sf.jga.fn.BinaryFunctor;
import net.sf.jga.fn.Generator;
import net.sf.jga.fn.UnaryFunctor;

/**
 * UnaryFunctor that uses a Generator to produce the 2nd argument to a given
 * BinaryFunctor.
 * <p>
 * Copyright &copy; 2004-2005  David A. Hall
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

public class Generate2nd<T1,T2,R> extends UnaryFunctor<T1,R> {
    
    static final long serialVersionUID = 8944237143829635388L;

    // The Functor applied to each value generated
    private BinaryFunctor<T1,T2,R> _fn;

    // The Nested Generator
    private Generator<T2> _gen;
    
    public Generate2nd(BinaryFunctor<T1,T2,R> fn, Generator<T2> gen) {
        _fn = fn;
        _gen = gen;
    }

    /**
     * Returns the Functor that is invoked
     */

    public BinaryFunctor<T1,T2,R> getFunctor() { return _fn; }

    /**
     * Returns the generator that produces the 2nd argument.
     */

    public Generator<T2> getGenerator() { return _gen; }
    
    /**
     * Returns the results of the functor, using the nested generator to
     * produce the first argument.
     */
    public R fn(T1 arg) {
        return _fn.fn(arg, _gen.fn());
    }

    /**
     * Calls the Visitor's <code>visit(Generate2nd)</code> method, if it
     * implements the nested Visitor interface.
     */
    public void accept(net.sf.jga.fn.Visitor v) {
        if (v instanceof Generate2nd.Visitor)
            ((Generate2nd.Visitor)v).visit(this);
        else
            v.visit(this);
    }

    // Object overrides

    public String toString() {
        return _fn+".generate2nd("+_gen+")"; 
    }
    
    // Acyclic Visitor
    
    /**
     * Interface for classes that may interpret a <b>Generate2nd</b>
     * functor.
     */
    public interface Visitor extends net.sf.jga.fn.Visitor {
        public void visit(Generate2nd<?,?,?> host);
    }
}
