// ============================================================================
// $Id: Count.java,v 1.16 2009/01/19 03:37:21 davidahall Exp $
// Copyright (c) 2003-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.algorithm;

import java.util.Iterator;
import net.sf.jga.fn.UnaryFunctor;
import net.sf.jga.fn.comparison.Equality;
import net.sf.jga.fn.comparison.EqualTo;
import net.sf.jga.util.FindIterator;

/**
 * Counts values in an iteration.
 * <p>
 * Copyright &copy; 2003-2005  David A. Hall
 *
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 * @deprecated This functionality is moved into {@link net.sf.jga.algorithms.Summarize} 
 */

public class Count<T> extends UnaryFunctor<Iterator<? extends T>,Long> {

    static final long serialVersionUID = 5477750093695096889L;    
    
    // the predicate used to determine if an element should be counted
    private UnaryFunctor<T,Boolean> _eq;

    /**
     * Builds a Count functor that counts instances of a specific value in an
     * iteration, using the value's equals() method.
     */
    public Count (T value) {
        this(new EqualTo<T>().bind2nd(value));
    }
    
    /**
     * Builds a Count functor that counts instances of a specific value in an
     * iteration, using the given Equality predicate.
     */
    public Count (Equality<T> eq, T value) {
        this(eq.bind2nd(value));
    }

    /**
     * Builds a Count functor that counts instances in an iteration for which
     * the given predicate is true
     */
    public Count (UnaryFunctor<T,Boolean> eq) {
        _eq = eq;
    }

    /**
     * Returns the predicate used to determine if an element should be counted.
     */
    public UnaryFunctor<T,Boolean> getComparisonFn() {
        return _eq;
    }

    /**
     * Returns the number of elements in the iteration that meet the given
     * criteria.
     */
    public Long fn (Iterator<? extends T> iterator) {
        long count = 0;
        FindIterator<T> finder = new FindIterator<T>(iterator);
        
        while (finder.findNext(_eq)) {
            ++count;
            finder.next();
        }
     
        return new Long(count);
    }

    /**
     * Calls the Visitor's <code>visit(Count)</code> method, if it
     * implements the nested Visitor interface.
     */
    public void accept(net.sf.jga.fn.Visitor v) {
        if (v instanceof Count.Visitor)
            ((Count.Visitor)v).visit(this);
        else
            v.visit(this);
    }

    // Object overrides

    public String toString() {
        return "Count["+_eq+"]";
    }
    
    // AcyclicVisitor
    
    /**
     * Interface for classes that may interpret an <b>Count</b> functor.
     */
    public interface Visitor extends net.sf.jga.fn.Visitor {
        public void visit(Count<?> host);
    }
}
