// ============================================================================
// $Id: ForEach.java,v 1.14 2009/01/19 03:37:21 davidahall Exp $
// Copyright (c) 2003-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================
package net.sf.jga.fn.algorithm;

import java.util.Iterator;
import net.sf.jga.fn.UnaryFunctor;

/**
 * Applies a UnaryFunctor to each element in an iteration, and returns the
 * final result.  Each element in the iteration is passed to the functor in
 * turn.  The result of the final call to the functor is returned.  If the
 * iteration was empty, then the result of this function is null.
 * <p>
 * Copyright &copy; 2003-2005  David A. Hall
 *
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 * @deprecated This functionality is moved into {@link net.sf.jga.algorithms.Summarize} 
 */

public class ForEach<T,R> extends UnaryFunctor<Iterator<? extends T>, R> {
    static final long serialVersionUID = -2342252375909337974L;

    // The functor to be applied
    private UnaryFunctor<T,R> _fn;

    /**
     * Builds a ForEach functor that will use the given functor to process
     * elements in an iteration.
     * @throws IllegalArgumentException if the functor is null
     */
    public ForEach(UnaryFunctor<T,R> fn) {
        if (fn == null)
            throw new IllegalArgumentException();
        
        _fn = fn;
    }

    /**
     * Returns the functor used to process elements in an iteration.
     */
    public UnaryFunctor<T,R> getFunction() {
        return _fn;
    }

    /**
     * Apply the functor to each element in the iteration and return the final
     * result.
     * @return the result of the last execution of the functor, or null if the
     * functor is not executed.
     */
    public R fn(Iterator<? extends T> iterator) {
        R value = null;
        while (iterator.hasNext()) {
            value = _fn.fn(iterator.next());
        }

        return value;
    }
    
    /**
     * Calls the Visitor's <code>visit(ForEach)</code> method, if it
     * implements the nested Visitor interface.
     */
    public void accept(net.sf.jga.fn.Visitor v) {
        if (v instanceof ForEach.Visitor)
            ((ForEach.Visitor)v).visit(this);
        else
            v.visit(this);
    }

    // Object overrides

    public String toString() {
        return "ForEach";
    }
    
    // AcyclicVisitor
    
    /**
     * Interface for classes that may interpret an <b>ForEach</b> functor.
     */
    public interface Visitor extends net.sf.jga.fn.Visitor {
        public void visit(ForEach<?,?> host);
    }
}
