// ============================================================================
// $Id: MinValue.java,v 1.10 2009/01/19 03:37:21 davidahall Exp $
// Copyright (c) 2003-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================
package net.sf.jga.fn.algorithm;

import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import net.sf.jga.fn.UnaryFunctor;

/**
 * Identifies the smallest value in a collection.
 * <p>
 * To serialize a MinValue, the comparator passed at construction must be
 * Serializable.
 * <p>
 * Copyright &copy; 2003-2005  David A. Hall
 *
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 * @deprecated This functionality is moved into {@link net.sf.jga.algorithms.Summarize} 
 */

public class MinValue<T> extends UnaryFunctor<Collection<? extends T>, T> {
    
    static final long serialVersionUID = 8518667936731775433L;

    private Comparator<T> _comp;

    /**
     * Builds a MinValue functor that will use the given comparator to
     * compare elements in the collection.  Typically, the functor would compare
     * its two arguments and return the lesser value.
     * @throws IllegalArgumentException if the Comparator is null
     */
    public MinValue(Comparator<T> comp) {
        if (comp == null)
            throw new IllegalArgumentException();
        
        _comp = comp;
    }

    /**
     * Returns the functor used to order values in the collection.
     */
    public Comparator<T> getComparator() {
        return _comp;
    }

    /**
     * Return the smallest value in the collection
     * @throws NoSuchElementException if the collection is empty
     */
    public T fn(Collection<? extends T> collection) {
        return Collections.min(collection, _comp);
    }
    
    /**
     * Calls the Visitor's <code>visit(MinValue)</code> method, if it
     * implements the nested Visitor interface.
     */
    public void accept(net.sf.jga.fn.Visitor v) {
        if (v instanceof MinValue.Visitor)
            ((MinValue.Visitor)v).visit(this);
        else
            v.visit(this);
    }

    // Object overrides

    public String toString() {
        return "MinValue";
    }
    
    // AcyclicVisitor
    
    /**
     * Interface for classes that may interpret an <b>MinValue</b> functor.
     */
    public interface Visitor extends net.sf.jga.fn.Visitor {
        public void visit(MinValue<?> host);
    }
}
