// ============================================================================
// $Id: ArithmeticFunctors.java,v 1.2 2006/12/16 16:48:58 davidahall Exp $
// Copyright (c) 2006  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.arithmetic;

import java.util.Iterator;
import net.sf.jga.fn.BinaryFunctor;
import net.sf.jga.fn.UnaryFunctor;

/**
 * Static factory methods for the functors in the Comparison package.
 * <p>
 * Copyright &copy; 2006  David A. Hall
 */

// This class suppresses unchecked cast warnings in numerous methods that take a generic
// value of type T, call getClass(), then cast the class to a Class<T>.  I believe that
// the warning is technically correct, but the way these methods are written, you'd have
// to be pretty abusive to trigger the error condition.

public final class ArithmeticFunctors {
    /**
     */
    static public <T extends Number> UnaryFunctor<Iterator<T>,T> average(Class<T> t) {
        return new Average<T>(t);
    }
    
//     /**
//      */
//     static public <T extends Number> BinaryFunctor<T,T,T> and(Class<T> t) {
//         return new BitwiseAnd<T>(t);
//     }
    
//     /**
//      */
//     @SuppressWarnings("unchecked") 
//     static public <T extends Number> UnaryFunctor<T,T> and(T value) {
//         return new BitwiseAnd<T>((Class<T>) value.getClass()).bind2nd(value);
//     }
        
    
    /**
     */
    static public <T extends Number> BinaryFunctor<T,T,T> bitwiseAnd(Class<T> t) {
        return new BitwiseAnd<T>(t);
    }
        
    /**
     */
    @SuppressWarnings("unchecked") 
    static public <T extends Number> UnaryFunctor<T,T> bitwiseAnd(T value) {
        return new BitwiseAnd<T>((Class<T>) value.getClass()).bind2nd(value);
    }
        
    /**
     */
    static public <T extends Number> UnaryFunctor<T,T> bitwiseNot(Class<T> t) {
        return new BitwiseNot<T>(t);
    }
        
    /**
     */
    static public <T extends Number> BinaryFunctor<T,T,T> bitwiseOr(Class<T> t) {
        return new BitwiseOr<T>(t);
    }

    /**
     */
    @SuppressWarnings("unchecked") 
    static public <T extends Number> UnaryFunctor<T,T> bitwiseOr(T value) {
        return new BitwiseOr<T>((Class<T>) value.getClass()).bind2nd(value);
    }

    /**
     */
    static public <T extends Number> BinaryFunctor<T,T,T> bitwiseXor(Class<T> t) {
        return new BitwiseXor<T>(t);
    }
        
    /**
     */
    @SuppressWarnings("unchecked") 
    static public <T extends Number> UnaryFunctor<T,T> bitwiseXor(T value) {
        return new BitwiseXor<T>((Class<T>) value.getClass()).bind2nd(value);
    }
        
    /**
     */
    static public <T extends Number> BinaryFunctor<T,T,T> divides(Class<T> t) {
        return new Divides<T>(t);
    }

    /**
     */
    @SuppressWarnings("unchecked") 
    static public <T extends Number> UnaryFunctor<T,T> divides(T value) {
        // Unchecked warning
        return new Divides<T>((Class<T>) value.getClass()).bind2nd(value); 
    }

    /**
     */
    static public <T extends Number> BinaryFunctor<T,T,T> minus(Class<T> t) {
        return new Minus<T>(t);
    }

    /**
     */
    @SuppressWarnings("unchecked") 
    static public <T extends Number> UnaryFunctor<T,T> minus(T value) {
        return new Minus<T>((Class<T>) value.getClass()).bind2nd(value);
    }
    
    /**
     */
    static public <T extends Number> BinaryFunctor<T,T,T> modulus(Class<T> t) {
        return new Modulus<T>(t);
    }

    /**
     */
    @SuppressWarnings("unchecked") 
    static public <T extends Number> UnaryFunctor<T,T> modulus(T value) {
        return new Modulus<T>((Class<T>) value.getClass()).bind2nd(value);
    } 

    /**
     */
    static public <T extends Number> BinaryFunctor<T,T,T> multiplies(Class<T> t) {
        return new Multiplies<T>(t);
    }

    /**
     */
    @SuppressWarnings("unchecked") 
    static public <T extends Number> UnaryFunctor<T,T> multiplies(T value) {
        return new Multiplies<T>((Class<T>) value.getClass()).bind2nd(value);
    } 

    /**
     */
    static public <T extends Number> UnaryFunctor<T,T> negate(Class<T> t) {
        return new Negate<T>(t);
    }

//     /**
//     /**
//      */
//     static public <T extends Number> UnaryFunctor<T,T> not(Class<T> t) {
//         return new BitwiseNot<T>(t);
//     }
    
//      */
//     static public <T extends Number> BinaryFunctor<T,T,T> or(Class<T> t) {
//         return new BitwiseOr<T>(t);
//     }
    
//     /**
//      */
//     @SuppressWarnings("unchecked") 
//     static public <T extends Number> UnaryFunctor<T,T> or(T value) {
//         return new BitwiseOr<T>((Class<T>) value.getClass()).bind2nd(value);
//     }

    /**
     */
    static public <T extends Number> BinaryFunctor<T,T,T> plus(Class<T> t) {
        return new Plus<T>(t);
    }

    /**
     */
    @SuppressWarnings("unchecked") 
    static public <T extends Number> UnaryFunctor<T,T> plus(T value) {
        return new Plus<T>((Class<T>) value.getClass()).bind2nd(value);
    }

    /**
     */
    static public <T extends Number> BinaryFunctor<T,Integer,T> shiftLeft(Class<T> t) {
        return new ShiftLeft<T>(t);
    }


    /**
     */
    static public <T extends Number> BinaryFunctor<T,Integer,T> shiftRight(Class<T> t) {
        return new ShiftRight<T>(t);
    }
    
    /**
     */
    static public <T extends Number> BinaryFunctor<T,Integer,T> unsignedShiftRight(Class<T> t) {
        return new UnsignedShiftRight<T>(t);
    }

    /**
     */
    static public <T extends Number,R extends Number> UnaryFunctor<T,R> valueOf(Class<R> r) {
        return new ValueOf<T,R>(r);
    }
    
//     /**
//      */
//     static public <T extends Number> BinaryFunctor<T,T,T> xor(Class<T> t) {
//         return new BitwiseXor<T>(t);
//     }
    
//     /**
//      */
//     @SuppressWarnings("unchecked") 
//     static public <T extends Number> UnaryFunctor<T,T> xor(T value) {
//         return new BitwiseXor<T>((Class<T>) value.getClass()).bind2nd(value);
//     }        
}

