// ============================================================================
// $Id: Average.java,v 1.4 2009/01/19 03:37:21 davidahall Exp $
// Copyright (c) 2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.arithmetic;

import java.text.MessageFormat;
import java.util.Iterator;
import net.sf.jga.fn.UnaryFunctor;

/**
 * Averages numeric values in an iteration.
 * <p>
 * Copyright &copy; 2005  David A. Hall
 *
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

public class Average<T extends Number> extends UnaryFunctor<Iterator<T>,T> {

    static final long serialVersionUID = -8341653586442484621L;
    
    // Utility class that performs the addition as appropriate for the
    // class given at construction
    private Arithmetic<T> _math;

    /**
     * Builds a functor that averages numeric values in an iteration
     */
    public Average(Class<T> type) {
         _math = ArithmeticFactory.getArithmetic(type);
         if (_math == null) {
             String msg = "No implementation of Arithmetic registered for {0}";
             throw new IllegalArgumentException(MessageFormat.format(msg, new Object[]{type}));
         }
    }
    
    /**
     * Returns the average of the elements in the iteration
     */
    public T fn (Iterator<T> iterator) {
        int count = 0;
        T sum = _math.zero();
        
        while (iterator.hasNext()) {
            T value = iterator.next();
            sum = _math.plus(sum, value);
            ++count;
        }
     
        return _math.divides(sum, _math.valueOf(count));
    }

    /**
     * Calls the Visitor's <code>visit(Average)</code> method, if it
     * implements the nested Visitor interface.
     */
    public void accept(net.sf.jga.fn.Visitor v) {
        if (v instanceof Average.Visitor)
            ((Average.Visitor)v).visit(this);
        else
            v.visit(this);
    }

    // Object overrides

    public String toString() {
        return "Average[]";
    }
    
    // AcyclicVisitor
    
    /**
     * Interface for classes that may interpret an <b>Average</b> functor.
     */
    public interface Visitor extends net.sf.jga.fn.Visitor {
        public void visit(Average<? extends Number> host);
    }
}
