// ============================================================================
// $Id: BigIntMath.java,v 1.7 2005/08/02 23:45:05 davidahall Exp $
// Copyright (c) 2003-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.arithmetic;

import java.math.BigInteger;

/**
 * Provides Arithmetic implementation for BigIntegers
 * <p>
 * Copyright &copy; 2003-2005  David A. Hall
 *
 * @author <a href="mailto:davidahall@users.sourceforge.net">David A. Hall</a>
 */

class BigIntMath implements IntegerArithmetic<BigInteger> {
    
    static final long serialVersionUID = 8004298803104674966L;
    
    /**
     * Returns the given value in the appropriate type
     * @throws IllegalArgumentException if the value cannot be converted
     */

    public BigInteger valueOf(Number value) throws IllegalArgumentException {
        return BigInteger.valueOf(value.longValue());
    }

    /**
     * Returns the value 0 of the appropriate type
     */

    public BigInteger zero() {
        return BigInteger.ZERO;
    }

    /**
     * Returns the value 1 of the appropriate type
     */

    public BigInteger one() {
        return BigInteger.ONE;
    }

   /**
    * For numeric arguments x and y, returns x + y
    * @return the sum of the two arguments
    */

    public BigInteger plus (BigInteger x, BigInteger y) {
        return x.add(y);
    }

   /**
    * For numeric arguments x and y, returns x - y
    * @return the difference of the two arguments
    */
     
    public BigInteger minus (BigInteger x, BigInteger y) {
        return x.subtract(y);
    }
     
   /**
    * For numeric arguments x and y, returns x * y
    * @return the product of the two arguments
    */

    public BigInteger multiplies (BigInteger x, BigInteger y){
        return x.multiply(y);
    }
         
    /**
     * For numeric arguments x and y, returns x / y 
     * @return the quotient of the two arguments
     */

    public BigInteger divides (BigInteger x, BigInteger y) {
        return x.divide(y);
    }
     
    /**
     * for numeric argument x, returns -x
     * @return the negative of its argument
     */

    public BigInteger negate (BigInteger x) {
        return x.negate();
    }
    
   /**
    * For numeric arguments x and y, returns x % y
    * @return the modulus of the two arguments
    */
     
    public BigInteger modulus (BigInteger x, BigInteger y) {
        return x.mod(y);
    }

    /**
     * For numeric arguments x and y, returns x &amp; y
     * @return x amp; y
     */

    public BigInteger and (BigInteger x, BigInteger y) {
        return x.and(y);
    }

    /**
     * For numeric arguments x and y, returns x | y
     * @return x | y
     */

    public BigInteger or (BigInteger x, BigInteger y) {
        return x.or(y);
    }

    /**
     * For numeric arguments x and y, returns x ^ y
     * @return x ^ y
     */

    public BigInteger xor (BigInteger x, BigInteger y) {
        return x.xor(y);
    }

    /**
     * For numeric argument x, returns ~x
     * @return the one's complement of the argument
     */

    public BigInteger not (BigInteger x) {
        return x.not();
    }
    
    /**
     * @return x << y
     */

    public BigInteger shiftLeft(BigInteger x, Integer y) {
        return x.shiftLeft(y.intValue());
    }        

    /**
     * @return x >> y
     */

    public BigInteger signedShiftRight(BigInteger x, Integer y) {
        return x.shiftRight(y.intValue());
    }

    /**
     * Optional.
     * @return x >>> y
     */

    public BigInteger unsignedShiftRight(BigInteger x, Integer y) {
        String msg = "BigInteger cannot support unsigned shift";
        throw new UnsupportedOperationException(msg);
    }
}
