// ============================================================================
// $Id: ByteMath.java,v 1.8 2005/08/02 23:45:05 davidahall Exp $
// Copyright (c) 2003-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.arithmetic;

/**
 * Provides Arithmetic implementation for Bytes
 * <p>
 * Copyright &copy; 2003-2005  David A. Hall
 * 
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

class ByteMath implements IntegerArithmetic<Byte> {
    
    static final long serialVersionUID = -8417130628589605890L;
    
    static public final Byte ZERO = new Byte((byte) 0);
    static public final Byte ONE  = new Byte((byte) 1);
    
    /**
     * Returns the given value in the appropriate type
     * @throws IllegalArgumentException if the value cannot be converted
     */

    public Byte valueOf(Number value) throws IllegalArgumentException {
        return new Byte(value.byteValue());
    }
    
    /**
     * Returns the value 0 of the appropriate type
     */

    public Byte zero() {
        return ZERO;
    }

    /**
     * Returns the value 1 of the appropriate type
     */

    public Byte one() {
        return ONE;
    }

   /**
    * For numeric arguments x and y, returns x + y
    * @return the sum of the two arguments
    */

    public Byte plus (Byte x, Byte y) {
        return new Byte((byte) (x.byteValue() + y.byteValue()));
    }

   /**
    * For numeric arguments x and y, returns x - y
    * @return the difference of the two arguments
    */
     
    public Byte minus (Byte x, Byte y) {
        return new Byte((byte) (x.byteValue() - y.byteValue()));
    }
     
   /**
    * For numeric arguments x and y, returns x * y
    * @return the product of the two arguments
    */

    public Byte multiplies (Byte x, Byte y){
        return new Byte((byte) (x.byteValue() * y.byteValue()));
    }
         

    /**
     * For numeric arguments x and y, returns x / y
     * @return the quotient of the two arguments
     */

    public Byte divides (Byte x, Byte y) {
        return new Byte((byte) (x.byteValue() / y.byteValue()));
    }
     
    /**
     * for numeric argument x, returns -x
     * @return the negative of its argument
     */

    public Byte negate (Byte x) {
        return new Byte((byte) -x.byteValue());
    }
    
   /**
    * For numeric arguments x and y, returns x % y
    * @return the modulus of the two arguments
    */
     
    public Byte modulus (Byte x, Byte y) {
        return new Byte((byte) (x.byteValue() % y.byteValue()));
    }

    /**
     * For numeric arguments x and y, returns x &amp; y
     * @return x amp; y
     */

    public Byte and (Byte x, Byte y) {
        return new Byte((byte) (x.byteValue() & y.byteValue()));
    }

    /**
     * For numeric arguments x and y, returns x | y
     * @return x | y
     */

    public Byte or (Byte x, Byte y) {
        return new Byte((byte) (x.byteValue() | y.byteValue()));
    }

    /**
     * For numeric arguments x and y, returns x ^ y
     * @return x ^ y
     */

    public Byte xor (Byte x, Byte y) {
        return new Byte((byte) (x.byteValue() ^ y.byteValue()));
    }

    /**
     * For numeric argument x, returns ~x
     * @return the one's complement of the argument
     */

    public Byte not (Byte x) {
        return new Byte((byte) ~(x.byteValue()));
    }

    /**
     * @return x << y
     */

    public Byte shiftLeft(Byte x, Integer y) {
        return new Byte((byte) (x.byteValue() << y.intValue()));
    }        

    /**
     * @return x >> y
     */

    public Byte signedShiftRight(Byte x, Integer y) {
        return new Byte((byte) (x.byteValue() >> y.intValue()));
    }

    /**
     * Optional.
     * @return x >>> y
     */

    public Byte unsignedShiftRight(Byte x, Integer y) {
        return new Byte((byte) (x.byteValue() >>> y.intValue()));
    }
}
