// ============================================================================
// $Id: DoubleMath.java,v 1.5 2005/08/02 23:45:06 davidahall Exp $
// Copyright (c) 2003-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================
/**
 * Provides Arithmetic implementation for Doubles
 * <p>
 * Copyright &copy; 2003-2005  David A. Hall
 * 
 * @author <a href="mailto:dave@dolphin.hallsquared.org">David A. Hall</a>
 */

package net.sf.jga.fn.arithmetic;

class DoubleMath implements Arithmetic<Double> {
     
    static final long serialVersionUID = 8842457834391281612L;

    static private final Double ZERO = new Double(0.0d);
    static private final Double ONE  = new Double(1.0d);
    
    /**
     * Returns the given value in the appropriate type
     * @throws IllegalArgumentException if the value cannot be converted
     */

    public Double valueOf(Number value) throws IllegalArgumentException {
        return new Double(value.doubleValue());
    }

    /**
     * Returns the value 0 of the appropriate type
     */

    public Double zero() {
        return ZERO;
    }

    /**
     * Returns the value 1 of the appropriate type
     */

    public Double one() {
        return ONE;
    }

   /**
    * For numeric arguments x and y, returns x + y
    * @return the sum of the two arguments
    */

    public Double plus (Double x, Double y) {
        return new Double(x.doubleValue() + y.doubleValue());
    }

   /**
    * For numeric arguments x and y, returns x - y
    * @return the difference of the two arguments
    */
     
    public Double minus (Double x, Double y) {
        return new Double(x.doubleValue() - y.doubleValue());
    }
     
   /**
    * For numeric arguments x and y, returns x * y
    * @return the product of the two arguments
    */
     

    public Double multiplies (Double x, Double y){
        return new Double(x.doubleValue() * y.doubleValue());
    }
         

    /**
     * for numeric argument x, returns -x
     * @return the negative of its argument
     */

    public Double negate (Double x) {
        return new Double(-x.doubleValue());
    }
    
    /**
     * For numeric arguments x and y, returns x / y
     * @return the quotient of the two arguments
     */

    public Double divides (Double x, Double y) {
        return new Double(x.doubleValue() / y.doubleValue());
    }
}
