// ============================================================================
// $Id: IntegerArithmetic.java,v 1.7 2005/08/02 23:45:06 davidahall Exp $
// Copyright (c) 2003-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.arithmetic;

/**
 * Defines arithmetic operations appropriate for Integral Numbers.
 * <p>
 * An implementation of IntegerArithmetic for classes (such as 
 * BigInteger) that provide the appropriate operations can simply map these
 * methods to the methods provided by the Number.  For the reference types,
 * the implementation will need to dereference the arguments, perform the
 * specified operation on the resulting primitives, and box up the result in
 * a new reference type.
 * <p>
 * Implementations of IntegerArithmetic for user-defined Number classes must
 * be registered with the ArithmeticFactory class.  See the Arithmetic class
 * for details.
 * <p>
 * Copyright &copy; 2003-2005  David A. Hall
 * 
 * @author <a href="mailto:davidahall@users.sourceforge.net">David A. Hall</a>
 */

public interface IntegerArithmetic<T extends Number>
     extends Arithmetic<T>
{
     /**
      * For numeric arguments x and y, returns x % y
      * @return the modulus of the two arguments
      */

     public T modulus (T x, T y);

     /**
      * For numeric arguments x and y, returns x & y
      * @return x & y
      */

     public T and (T x, T y);

    /**
      * For numeric arguments x and y, returns x | y
      * @return x | y
      */

     public T or (T x, T y);

    /**
      * For numeric arguments x and y, returns x ^ y
      * @return x ^ y
      */

     public T xor (T x, T y);

     /**
      * For numeric arguments x, returns ~x
      * @return the one's complement of the argument
      */

     public T not (T x);

    /**
     * @return x << y
     */

    public T shiftLeft(T x, Integer y);

    /**
     * @return x >> y
     */

    public T signedShiftRight(T x, Integer y);

    /**
     * Optional.
     * @return x >>> y
     */

    public T unsignedShiftRight(T x, Integer y);
}
