// ============================================================================
// $Id: LongMath.java,v 1.8 2005/08/02 23:45:06 davidahall Exp $
// Copyright (c) 2003-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.arithmetic;

/**
 * Provides an implementation if IntegerArithmetic for Longs
 * <p>
 * Copyright &copy; 2003-2005  David A. Hall
 * 
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

class LongMath implements IntegerArithmetic<Long> {
     
    static final long serialVersionUID = 7284047144544492350L;

    static private final Long ZERO = new Long(0L);
    static private final Long ONE  = new Long(1L);
    
    /**
     * Returns the given value in the appropriate type
     * @throws IllegalArgumentException if the value cannot be converted
     */

    public Long valueOf(Number value) throws IllegalArgumentException {
        return new Long(value.longValue());
    }

    /**
     * Returns the value 0 of the appropriate type
     */

    public Long zero() {
        return ZERO;
    }

    /**
     * Returns the value 1 of the appropriate type
     */

    public Long one() {
        return ONE;
    }

   /**
    * For numeric arguments x and y, returns x + y
    * @return the sum of the two arguments
    */

    public Long plus (Long x, Long y) {
        return new Long(x.longValue() + y.longValue());
    }

   /**
    * For numeric arguments x and y, returns x - y
    * @return the difference of the two arguments
    */
     
    public Long minus (Long x, Long y) {
        return new Long(x.longValue() - y.longValue());
    }
     
   /**
    * For numeric arguments x and y, returns x * y
    * @return the product of the two arguments
    */

    public Long multiplies (Long x, Long y){
        return new Long(x.longValue() * y.longValue());
    }
         

    /**
     * For numeric arguments x and y, returns x / y
     * @return the quotient of the two arguments
     */

    public Long divides (Long x, Long y) {
        return new Long(x.longValue() / y.longValue());
    }
     
    /**
     * for numeric argument x, returns -x
     * @return the negative of its argument
     */

    public Long negate (Long x) {
        return new Long(-x.longValue());
    }
    
   /**
    * For numeric arguments x and y, returns x % y
    * @return the modulus of the two arguments
    */
     
    public Long modulus (Long x, Long y) {
        return new Long(x.longValue() % y.longValue());
    }

    /**
     * For numeric arguments x and y, returns x &amp; y
     * @return x amp; y
     */

    public Long and (Long x, Long y) {
        return new Long(x.longValue() & y.longValue());
    }

    /**
     * For numeric arguments x and y, returns x | y
     * @return x | y
     */

    public Long or (Long x, Long y) {
        return new Long(x.longValue() | y.longValue());
    }

    /**
     * For numeric arguments x and y, returns x ^ y
     * @return x ^ y
     */

    public Long xor (Long x, Long y) {
        return new Long(x.longValue() ^ y.longValue());
    }

    /**
     * For numeric argument x, returns ~x
     * @return the one's complement of the argument
     */

    public Long not (Long x) {
        return new Long(~x.longValue());
    }

    /**
     * @return x << y
     */

    public Long shiftLeft(Long x, Integer y) {
        return new Long(x.longValue() << y.intValue());
    }        

    /**
     * @return x >> y
     */

    public Long signedShiftRight(Long x, Integer y) {
        return new Long(x.longValue() >> y.intValue());
    }

    /**
     * Optional.
     * @return x >>> y
     */

    public Long unsignedShiftRight(Long x, Integer y) {
        return new Long(x.longValue() >>> y.intValue());
    }
}
