// ============================================================================
// $Id: ShortMath.java,v 1.7 2005/08/02 23:45:06 davidahall Exp $
// Copyright (c) 2003-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.arithmetic;

/**
 * Provides Arithmetic implementation for Shorts
 * <p>
 * Copyright &copy; 2003-2005  David A. Hall
 * 
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

class ShortMath implements IntegerArithmetic<Short> {
     
    static final long serialVersionUID = 9167946402888608933L;

    static private final Short ZERO = new Short((short) 0);
    static private final Short ONE  = new Short((short) 1);
    
    /**
     * Returns the given value in the appropriate type
     * @throws IllegalArgumentException if the value cannot be converted
     */

    public Short valueOf(Number value) throws IllegalArgumentException {
        return new Short(value.shortValue());
    }

    /**
     * Returns the value 0 of the appropriate type
     */

    public Short zero() {
        return ZERO;
    }

    /**
     * Returns the value 1 of the appropriate type
     */

    public Short one() {
        return ONE;
    }

   /**
    * For numeric arguments x and y, returns x + y
    * @return the sum of the two arguments
    */

    public Short plus (Short x, Short y) {
        return new Short((short) (x.shortValue() + y.shortValue()));
    }

   /**
    * For numeric arguments x and y, returns x - y
    * @return the difference of the two arguments
    */
     
    public Short minus (Short x, Short y) {
        return new Short((short) (x.shortValue() - y.shortValue()));
    }
     
   /**
    * For numeric arguments x and y, returns x * y
    * @return the product of the two arguments
    */

    public Short multiplies (Short x, Short y){
        return new Short((short) (x.shortValue() * y.shortValue()));
    }
         

    /**
     * For numeric arguments x and y, returns x / y
     * @return the quotient of the two arguments
     */

    public Short divides (Short x, Short y) {
        return new Short((short) (x.shortValue() / y.shortValue()));
    }
     
    /**
     * for numeric argument x, returns -x
     * @return the negative of its argument
     */

    public Short negate (Short x) {
        return new Short((short) -x.shortValue());
    }
    
   /**
    * For numeric arguments x and y, returns x % y
    * @return the modulus of the two arguments
    */
     
    public Short modulus (Short x, Short y) {
        return new Short((short) (x.shortValue() % y.shortValue()));
    }
    /**
     * For numeric arguments x and y, returns x &amp; y
     * @return x amp; y
     */

    public Short and (Short x, Short y) {
        return new Short((short) (x.shortValue() & y.shortValue()));
    }

    /**
     * For numeric arguments x and y, returns x | y
     * @return x | y
     */

    public Short or (Short x, Short y) {
        return new Short((short) (x.shortValue() | y.shortValue()));
    }

    /**
     * For numeric arguments x and y, returns x ^ y
     * @return x ^y
     */

    public Short xor (Short x, Short y) {
        return new Short((short) (x.shortValue() ^ y.shortValue()));
    }

    /**
     * For numeric argument x, returns ~x
     * @return the one's complement of the argument
     */

    public Short not (Short x) {
        return new Short((short) ~x.shortValue());
    }

    
    /**
     * @return x << y
     */

    public Short shiftLeft(Short x, Integer y) {
        return new Short((short) (x.shortValue() << y.intValue()));
    }        

    /**
     * @return x >> y
     */

    public Short signedShiftRight(Short x, Integer y) {
        return new Short((short) (x.shortValue() >> y.intValue()));
    }

    /**
     * Optional.
     * @return x >>> y
     */

    public Short unsignedShiftRight(Short x, Integer y) {
        return new Short((short) (x.shortValue() >>> y.intValue()));
    }
}
