// ============================================================================
// $Id: ComparisonFunctors.java,v 1.7 2007/11/18 01:55:53 davidahall Exp $
// Copyright (c) 2006  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.comparison;

import java.util.Comparator;
import net.sf.jga.fn.BinaryFunctor;
import net.sf.jga.fn.UnaryFunctor;

/**
 * Static factory methods for the functors in the Comparison package.
 * <p>
 * Copyright &copy; 2006  David A. Hall
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

public final class ComparisonFunctors {
    /**
     */
    static public <T> UnaryFunctor<T,Boolean> between(T lo, T hi, Comparator<? super T> comp) {
        return new Between<T>(lo, hi, comp);
    }

    /**
     */
    static public <T extends Comparable<? super T>> UnaryFunctor<T,Boolean>
    between(T lo, T hi)
    {
        return new Between.Comparable<T>(lo, hi);
    }

    /**
     */
    static public <T> UnaryFunctor<T,Boolean>
    between(UnaryFunctor<T,Boolean> lo, UnaryFunctor<T,Boolean> hi)
    {
        return new Between<T>(lo, hi);
    }


    /**
     */
    static public <T> BinaryFunctor<T,T,Integer> comparatorFn(Comparator<T> comp) {
        return new ComparatorFn<T>(comp);
    }

    /**
     */
    static public <T> BinaryFunctor<T,T,Boolean> equalEqual() {
        return new EqualEqual<T>();
    }

    /**
     */
    static public <T> UnaryFunctor<T,Boolean> equalEqual(T value) {
        return ComparisonFunctors.<T>equalEqual().bind2nd(value);
    }

    /**
     */
    static public <T> BinaryFunctor<T,T,Boolean> equalTo(Class<T> hint) {
        return new EqualTo<T>();
    }

    /**
     */
    static public <T> BinaryFunctor<T,T,Boolean> equalTo(Comparator<? super T> comp) {
        return new EqualTo<T>(comp);
    }
    
    /**
     */
    static public <T> UnaryFunctor<T,Boolean> equalTo(T value) {
        return new EqualTo<T>().bind2nd(value);
    }

    /**
     */
    static public <T> UnaryFunctor<T,Boolean> equalTo(Comparator<? super T> comp, T value) {
        return new EqualTo<T>(comp).bind2nd(value);
    }
    
    /**
     */
    static public <T> UnaryFunctor<T,Boolean> equalTo(Equality<T> eq, T value) {
        return eq.bind2nd(value);
    }
    
    /**
     */
    static public <T> BinaryFunctor<T,T,Boolean> greater(Comparator<? super T> comp) {
        return new Greater<T>(comp);
    }

    /**
     */
    static public <T> UnaryFunctor<T,Boolean> greater(Comparator<? super T> comp, T value) {
        return ComparisonFunctors.<T>greater(comp).bind2nd(value);
    }

    /**
     */
    static public <T extends Comparable<? super T>>
    BinaryFunctor<T,T,Boolean> greater(Class<T> hint)
    {
        return new Greater.Comparable<T>();
    }

    /**
     */
    static public <T extends Comparable<? super T>> UnaryFunctor<T,Boolean>
    greater(T value)
    {
        return new Greater.Comparable<T>().bind2nd(value);
    }

    /**
     */
    static public <T> BinaryFunctor<T,T,Boolean> greaterEqual(Comparator<? super T> comp) {
        return new GreaterEqual<T>(comp);
    }

    /**
     */
    static public <T> UnaryFunctor<T,Boolean> greaterEqual(Comparator<? super T> comp, T value) {
        return ComparisonFunctors.<T>greaterEqual(comp).bind2nd(value);
    }

    /**
     */
    static public <T extends Comparable<? super T>> BinaryFunctor<T,T,Boolean>
    greaterEqual(Class<T> hint)
    {
        return new GreaterEqual.Comparable<T>();
    }

    /**
     */
    static public <T extends Comparable<? super T>> UnaryFunctor<T,Boolean>
    greaterEqual(T value)
    {
        return new GreaterEqual.Comparable<T>().bind2nd(value);
    }

    /**
     */      
    static public <T> UnaryFunctor<T,Boolean> isNull() {
        return new EqualTo<T>().bind2nd(null);
    }

    /**
     */
    static public <T> UnaryFunctor<T,Boolean> notNull() {
        return new NotEqualTo<T>().bind2nd(null);
    }

    /**
     */
    static public <T> BinaryFunctor<T,T,Boolean> less(Comparator<? super T> comp) {
        return new Less<T>(comp);
    }

    /**
     */
    static public <T> UnaryFunctor<T,Boolean> less(Comparator<? super T> comp, T value) {
        return ComparisonFunctors.<T>less(comp).bind2nd(value);
    }

    /**
     */
    static public <T extends Comparable<? super T>> BinaryFunctor<T,T,Boolean>
    less(Class<T> hint)
    {
        return new Less.Comparable<T>();
    }

    /**
     */
    static public <T extends Comparable<? super T>> UnaryFunctor<T,Boolean>
    less(T value)
    {
        return new Less.Comparable<T>().bind2nd(value);
    }

    /**
     */
    static public <T> BinaryFunctor<T,T,Boolean> lessEqual(Comparator<? super T> comp) {
        return new LessEqual<T>(comp);
    }

    /**
     */
    static public <T> UnaryFunctor<T,Boolean> lessEqual(Comparator<? super T> comp, T value) {
        return ComparisonFunctors.<T>lessEqual(comp).bind2nd(value);
    }

    /**
     */
    static public <T extends Comparable<? super T>> BinaryFunctor<T,T,Boolean>
    lessEqual(Class<T> hint)
    {
        return new LessEqual.Comparable<T>();
    }

    /**
     */
    static public <T extends Comparable<? super T>> UnaryFunctor<T,Boolean>
    lessEqual(T value)
    {
        return new LessEqual.Comparable<T>().bind2nd(value);
    }

    /**
     */
    static public <T> BinaryFunctor<T,T,T> max(Comparator<? super T> comp) {
        return new Max<T>(comp);
    }
    
    /**
     */
    static public <T extends Comparable<? super T>> BinaryFunctor<T,T,T>
    max(Class<T> hint)
    {
        return new Max.Comparable<T>();
    }

    /**
     */
    static public <T> BinaryFunctor<T,T,T> min(Comparator<? super T> comp) {
        return new Min<T>(comp);
    }
    
    /**
     */
    static public <T extends Comparable<? super T>> BinaryFunctor<T,T,T>
    min(Class<T> hint)
    {
        return new Min.Comparable<T>();
    }

    /**
     */
    static public <T> BinaryFunctor<T,T,Boolean> notEqualEqual() {
        return new NotEqualEqual<T>();
    }

    /**
     */
    static public <T> UnaryFunctor<T,Boolean> notEqualEqual(T value) {
        return new NotEqualEqual<T>().bind2nd(value);
    }

    /**
     */
    static public <T> BinaryFunctor<T,T,Boolean> notEqualTo(Class<T> hint) {
        return new NotEqualTo<T>();
    }

    /**
     */
    static public <T> BinaryFunctor<T,T,Boolean> notEqualTo(Comparator<? super T> comp) {
        return new NotEqualTo<T>(comp);
    }
    
    /**
     */
    static public <T> UnaryFunctor<T,Boolean> notEqualTo(T value) {
        return new NotEqualTo<T>().bind2nd(value);
    }

    /**
     */
    static public <T> UnaryFunctor<T,Boolean> notEqualTo(Comparator<? super T> comp, T value) {
        return new NotEqualTo<T>(comp).bind2nd(value);
    }
}

