// ============================================================================
// $Id: EqualEqual.java,v 1.16 2009/01/19 03:37:21 davidahall Exp $
// Copyright (c) 2002-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.comparison;

/**
 * Binary Predicate that returns TRUE for object arguments <b>x</b> and
 * <b>y</b> when x == y using the built-in == operator.
 * <p>
 * Copyright &copy; 2002-2005  David A. Hall
 *
 * @author <a href="mailto:davidahall@users.sourceforge.net">David A. Hall</a>
 **/

public class EqualEqual<T> extends Equality<T> {

    static final long serialVersionUID = -3103153568586667984L;

    // BinaryPredicate interface
    
    /**
     * Given arguments <b>x</b> and <b>y</b>, returns x == y.
     * 
     * @return x == y
     */
    public Boolean fn(T x, T y) {
        return (x == y);
    }

    /**
     * Calls the Visitor's <code>visit(EqualEqual)</code> method, if it
     * implements the nested Visitor interface.
     */
    public void accept(net.sf.jga.fn.Visitor v) {
        if (v instanceof EqualEqual.Visitor)
            ((EqualEqual.Visitor)v).visit(this);
        else
            v.visit(this);
    }

    // Object overrides

    public String toString() {
        return "EqualEqual";
    }
    
    // Acyclic Visitor
    
    /**
     * Interface for classes that may interpret an <b>EqualEqual</b> predicate.
     */
    public interface Visitor extends net.sf.jga.fn.Visitor {
        public void visit(EqualEqual<?> host);
    }
}
