// ============================================================================
// $Id: LogicalFunctors.java,v 1.4 2008/02/08 04:22:34 davidahall Exp $
// Copyright (c) 2006  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.logical;

import java.util.Arrays;
import java.util.Collection;
import net.sf.jga.fn.BinaryFunctor;
import net.sf.jga.fn.UnaryFunctor;

/**
 * Static factory methods for the functors in the Logical package.
 * <p>
 * Copyright &copy; 2006  David A. Hall
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

public final class LogicalFunctors {

    static private LogicalAnd _andInstance = new LogicalAnd();
    static private LogicalNot _notInstance = new LogicalNot();
    static private LogicalOr _orInstance = new LogicalOr();

    /**
     */
    static public <T> UnaryFunctor<T,Boolean> all() {
        return new All<T>();
    }
    
    /**
     */
    static public <T> UnaryFunctor<T,Boolean> all(Collection<UnaryFunctor<T,Boolean>> branches) {
        return new All<T>(branches);
    }
    
    /**
     */
    static public <T> UnaryFunctor<T,Boolean> all(UnaryFunctor<T,Boolean>... branches) {
        return new All<T>(Arrays.asList(branches));
    }

//     /**
//      */
//     static public BinaryFunctor<Boolean,Boolean,Boolean> and() {
//         return logicalAnd();
//     }

    
    /**
     */
    static public <T> UnaryFunctor<T,Boolean> any() {
        return new Any<T>();
    }
    
    /**
     */
    static public <T> UnaryFunctor<T,Boolean> any(Collection<UnaryFunctor<T,Boolean>> branches) {
        return new Any<T>(branches);
    }
    
    /**
     */
    static public <T> UnaryFunctor<T,Boolean> any(UnaryFunctor<T,Boolean>... branches) {
        return new Any<T>(Arrays.asList(branches));
    }
    
    /**
     */
    static public <T1,T2> BinaryFunctor<T1,T2,Boolean>
    binaryNegate(BinaryFunctor<? super T1,? super T2,Boolean> p)
    {
        return new BinaryNegate<T1,T2>(p);
    }

    /**
     */
    static public BinaryFunctor<Boolean,Boolean,Boolean> logicalAnd() {
        return _andInstance;
    }

    /**
     */
    static public BinaryFunctor<Boolean,Boolean,Boolean> logicalOr() {
        return _orInstance;
    }

    /**
     */
    static public UnaryFunctor<Boolean,Boolean> logicalNot() {
        return _notInstance;
    }
    
//     /**
//      */
//     static public <T1,T2> BinaryFunctor<T1,T2,Boolean>
//     negate(BinaryFunctor<? super T1, ? super T2, Boolean> p)
//     {
//         return binaryNegate(p);
//     }

//     /**
//      */
//     static public <T> UnaryFunctor<T,Boolean> negate(UnaryFunctor<? super T, Boolean> p) {
//         return unaryNegate(p);
//     }
 
//     /**
//      */
//     static public UnaryFunctor<Boolean,Boolean> not() {
//         return logicalNot();
//     }
 
//     /**
//      */
//     static public BinaryFunctor<Boolean,Boolean,Boolean> or() {
//         return logicalOr();
//     }
    
    /**
     */
    static public <T> UnaryFunctor<T,Boolean> unaryNegate(UnaryFunctor<? super T, Boolean> p) {
        return new UnaryNegate<T>(p);
    }
}
