// ============================================================================
// $Id: LogicalOr.java,v 1.13 2007/02/07 12:20:48 davidahall Exp $
// Copyright (c) 2002-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.logical;

import net.sf.jga.fn.BinaryFunctor;

/**
 * Binary Predicate that returns true when either of Boolean arguments <b>x</b>
 * and <b>y</b> are true.
 * <p>
 * Note that this functor does <i>not</i> short circuit the evaluation
 * of the second argument.  The reason for this is that, by itself,
 * this functor accepts only boolean arguments.  When it is used in
 * conjunction with the adaptor functors to implement compound
 * functors, the adaptor cannot know that the functor it is adapting
 * may not need to use both arguments, so the adaptor will fully
 * evaluate both arguments before passing them to the And functor.
 * This would give the illusion that the short circuit logic is
 * broken, when it is in fact unimplementable using the standard
 * functor mechanism.
 * <p>
 * Copyright &copy; 2002-2005  David A. Hall
 *
 * @author <a href="mailto:davidahall@users.sourceforge.net">David A. Hall</a>
 **/

public class LogicalOr extends BinaryFunctor<Boolean,Boolean,Boolean> {

    static final long serialVersionUID = -5329467559457020210L;
    
    // BinaryPredicate interface
    
    /**
     * Given Boolean arguments <b>x</b> and <b>y</b>, returns true when either
     * x and y are true, otherwise false.
     * 
     * @return x | y
     */
    public Boolean fn(Boolean x, Boolean y) {
        return x | y;
    }

    /**
     * Calls the Visitor's <code>visit(LogicalOr)</code> method, if it
     * implements the nested Visitor interface.
     */
    public void accept(net.sf.jga.fn.Visitor v) {
        if (v instanceof LogicalOr.Visitor)
            ((LogicalOr.Visitor)v).visit(this);
        else
            v.visit(this);
    }
    
    // Object overrides

    public String toString() {
        return "LogicalOr";
    }
    
    // AcyclicVisitor
    
    /**
     * Interface for classes that may interpret a <b>LogicalOr</b>
     * predicate.
     */
    public interface Visitor extends net.sf.jga.fn.Visitor {
        public void visit(LogicalOr bp);
    }
}
