// ============================================================================
// $Id: Cast.java,v 1.7 2009/01/19 03:37:21 davidahall Exp $
// Copyright (c) 2004-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.property;

import java.text.MessageFormat;
import net.sf.jga.fn.UnaryFunctor;

/**
 * UnaryFunctor that returns its argument, cast to the given type.  Executing
 * the functor will throw ClassCastExceptions if the argument cannot be cast
 * to the given type.
 * <p>
 * Copyright &copy; 2004-2005  David A. Hall
 *
 * @author <a href="mailto:davidahall@users.sourceforge.net">David A. Hall</a>
 **/

public class Cast<T,R> extends UnaryFunctor<T,R> {

    static final long serialVersionUID = 2712605844695159349L;

    private Class<R> _class;

    /**
     * Builds a Cast fuctor that returns values cast to the given type.
     * @throws IllegalArgumentException if the method name is null or empty, or
     *     if the argument type array is null.
     */
    public Cast(Class<R> cl) {
        if (cl == null)
            throw new IllegalArgumentException("A class must be given");
        
        _class = cl;
    }


    public Class<R> getCastClass() { return _class; }
    
    // UnaryPredicate interface
    
    @SuppressWarnings("unchecked")
    // In Java 1.5, we could write this as 'return _class.cast(arg);', but 
    // that isn't backwards compatable with 1.4, sowe'll do it another way.
    public R fn(T arg) {
        
        if (_class.isInstance(arg))
            return (R) arg;

        String msg = "Cannot cast [{0}] to class {1}";
        Object[] args = new Object[]{arg, _class.getName()};
        ClassCastException x = new ClassCastException(MessageFormat.format(msg, args)); 
        throw x;
    }

    /**
     * Calls the Visitor's <code>visit(InstanceOf)</code> method, if it
     * implements the nested Visitor interface.
     */
    public void accept(net.sf.jga.fn.Visitor v) {
        if (v instanceof Cast.Visitor)
            ((Cast.Visitor)v).visit(this);
        else
            v.visit(this);
    }
    
    // Object overrides

    public String toString() {
        return "Cast["+_class.getName()+"]";
    }
    
   // AcyclicVisitor
    
    /**
     * Interface for classes that may interpret a <b>Cast</b>
     * function.
     */
    public interface Visitor extends net.sf.jga.fn.Visitor {
        public void visit(Cast<?,?> host);
    }
}
