// ============================================================================
// $Id: PropertyFunctors.java,v 1.5 2009/08/14 12:18:53 davidahall Exp $
// Copyright (c) 2006  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.property;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import net.sf.jga.fn.BinaryFunctor;
import net.sf.jga.fn.UnaryFunctor;
import net.sf.jga.fn.Generator;
import net.sf.jga.fn.adaptor.Identity;

/**
 * Static factory methods for the functors in the Property package.
 * <p>
 * Copyright &copy; 2006  David A. Hall
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

public final class PropertyFunctors {
    /**
     */
    static public <T1,T2> BinaryFunctor<T1,T2,Object[]> arrayBinary() {
        return new ArrayBinary<T1,T2>();
    }

    
    /**
     */
    static public <T> UnaryFunctor<T,Object[]> arrayUnary() {
        return new ArrayUnary<T>();
    }

    
    /**
     */
    static public <T,R> UnaryFunctor<T,R> cast(Class<R> r) {
        return new Cast<T,R>(r);
    }

     
    /**
     */
    static public <T,V> UnaryFunctor<T,Boolean> compareProperty(Class<T> t, String name, V value) {
        return new CompareProperty<T,V>(t, name, value);
    }

    
    /**
     */
    static public <T,V> UnaryFunctor<T,Boolean>
    compareProperty(Class<T> t, String name, BinaryFunctor<V,V,Boolean> pred, V value)
    {
        return new CompareProperty<T,V>(t, name, pred, value);
    }

    
    /**
     */
    static public <R> UnaryFunctor<Object[],R> construct(Constructor<R> ctor) {
        return new Construct<R>(ctor);
    }

    
    /**
     */
    static public <R> UnaryFunctor<Object[],R> construct(Class<R> r, Class<?>... argclasses) {
        return new Construct<R>(r, argclasses);
    }

    
    /**
     */
    static public <R> Generator<R> constructDefault(Class<R> r) {
        return new ConstructDefault<R>(r);
    }

    
    /**
     */
    static public <T,R> UnaryFunctor<T,R> constructUnary(Class<R> r, Class<T> t) {
        return new ConstructUnary<T,R>(t,r);
    }

    
    /**
     */
    static public <T,R> UnaryFunctor<T,R> getField(Class<T> t,String name) {
        return new GetField<T,R>(t, name);
    }
    
    
    /**
     */
    static public <T,R> UnaryFunctor<T,R> getField(Class<T> t, Field field) {
        return new GetField<T,R>(t, field);
    }

    
    /**
     */
    static public <T,R> UnaryFunctor<T,R> getField(Field field) {
        return new GetField<T,R>(field);
    }

    
    /**
     */
    static public <T,R> UnaryFunctor<T,R> getProperty(Class<T> t,String name) {
        return new GetProperty<T,R>(t, name);
    }

    
    /**
     */
    static public <T,R> UnaryFunctor<T,R> getProperty(Class<T> t, String name, Class<R> hint) {
        return new GetProperty<T,R>(t,name);
    }
    
    
    /**
     */
    static public <T> UnaryFunctor<T,Boolean> instanceOf(Class<?> type) {
        return new InstanceOf<T>(type);
    }


    /**
     */
    static public <T1,T2,R> BinaryFunctor<T1,Object[],R> 
    invokeMethod(Class<T1> t1, String name, Class<T2> t2)
    {
        return new InvokeMethod<T1,R>(t1, name, t2);
    }

    
    /**
     */
    static public <T1,T2,R> BinaryFunctor<T1,T2,R>
    invokeOneArgMethod(Class<T1> t1, String name, Class<T2> t2)
    {
        return new InvokeMethod<T1,R>(t1, name, t2)
                    .distribute(new Identity<T1>(), new ArrayUnary<T2>());
    }

    
    /**
     */
    static public <T,R> BinaryFunctor<T,Object[],R> invokeMethod(Method method) {
        return new InvokeMethod<T,R>(method);
    }


    /**
     */
    static public <T,R> BinaryFunctor<T,Object[],R> invokeMethod(Class<T> t, Method method) {
        return new InvokeMethod<T,R>(t, method);
    }


    /**
     */
    static public <T,R> BinaryFunctor<T,Object[],R>
    invokeMethod(Class<T> t, String name, Class<?>... argtypes) {
        return new InvokeMethod<T,R>(t, name, argtypes);
    }


    /**
     */
    static public <T,R> UnaryFunctor<T,R> invokeNoArgMethod(Class<T> t, Method method) {
        return new InvokeNoArgMethod<T,R>(t, method);
    }


    /**
     */
    static public <T,R> UnaryFunctor<T,R> invokeNoArgMethod(Method method) {
        return new InvokeNoArgMethod<T,R>(method);
    }


    /**
     */
    static public <T,R> UnaryFunctor<T,R> invokeNoArgMethod(Class<T> t, String name) {
        return new InvokeNoArgMethod<T,R>(t, name);
    }


    /**
     */
    static public <T1,T2> BinaryFunctor<T1,T2,T2> setField(Field field) {
        return new SetField<T1,T2>(field);
    }
    
    /**
     */
    static public <T1,T2> BinaryFunctor<T1,T2,T2> setField(Class<T1> t1, Field field) {
        return new SetField<T1,T2>(t1, field);
    }
    
    /**
     */
    static public <T1,T2> BinaryFunctor<T1,T2,T2> setField(Class<T1> t1, Field field, Class<T2> t2) {
        return new SetField<T1,T2>(t1, field, t2);
    }


    /**
     */
    static public <T1,T2> BinaryFunctor<T1,T2,T2> setField(Class<T1> t1, String name, Class<T2> t2) {
        return new SetField<T1,T2>(t1, name, t2);
    }


    /**
     */
    static public <T1,T2> BinaryFunctor<T1,T2,T2> setProperty(Class<T1> t1,String name,Class<T2> t2){
        return new SetProperty<T1,T2>(t1, name, t2);
    }
}
