// ============================================================================
// $Id: StringFunctors.java,v 1.2 2006/11/30 05:05:23 davidahall Exp $
// Copyright (c) 2006  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.fn.string;

import java.text.DateFormat;
import java.text.Format;
import java.text.MessageFormat;
import java.text.NumberFormat;
import java.util.Date;
import java.util.regex.Pattern;
import net.sf.jga.fn.UnaryFunctor;

/**
 * Static factory methods for the functors in the String package.
 * <p>
 * Copyright &copy; 2006  David A. Hall
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

public final class StringFunctors {
    /**
     * Returns a functor that presents its argument in String form using the argument's
     * toString() method.
     */
    static public <T> UnaryFunctor<T,String> defaultFormat() {
        return new DefaultFormat<T>();
    }


    /**
     * Returns a functor that presents its argument in String form using the given format
     */
    static public UnaryFunctor<Object[],String> formatValue(MessageFormat format) {
        return new FormatValue<Object[]>(format);
    }

    
    /**
     * Returns a functor that presents its argument in String form using the given format
     */
    static public <T extends Date> UnaryFunctor<T,String> formatValue(DateFormat format) {
        return new FormatValue<T>(format);
    }

    
    /**
     * Returns a functor that presents its argument in String form using the given format
     */
    static public <T extends Number> UnaryFunctor<T,String> formatValue(NumberFormat format) {
        return new FormatValue<T>(format);
    }

    
    /**
     * Returns a functor that matches its String argument against the given regex
     */
    static public UnaryFunctor<String,Boolean> match(String regex) {
        return new Match(regex);
    }

    
    /**
     * Returns a functor that matches its String argument against the given pattern
     */
    static public UnaryFunctor<String,Boolean> match(Pattern pattern) {
        return new Match(pattern);
    }

    
    /**
     * Returns a functor that parses its argument using the given format and returns an object
     * of the given class.
     */
    static public <R> UnaryFunctor<String,R>
    parseFormat(Format format, UnaryFunctor<Object,R> converter)
    {
         return new ParseFormat<R>(format, converter);
    }

    
    /**
     * Returns a functor that parses its argument using the given format and returns an object
     * of the given class.
     */
    static public <R extends Date> UnaryFunctor<String,R>
    parseFormat(Class<R> dateClass, DateFormat format)
    {
        return new ParseFormat.Date<R>(dateClass, format);
    }

    
    /**
     * Returns a functor that parses its argument using the given format and returns an object
     * of the given class.
     */
    static public <R extends Number> UnaryFunctor<String,R>
    parseFormat(Class<R> numClass,NumberFormat format)
    {
        return new ParseFormat.Number<R>(numClass, format);
    }
} 
