// ============================================================================
// $Id: BinaryFunctorRef.java,v 1.10 2009/06/27 23:48:26 davidahall Exp $
// Copyright (c) 2004-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.parser;

import net.sf.jga.fn.BinaryFunctor;

/**
 * Contains and describes a BinaryFunctor being built by the functor parser.
 * <p>
 * Copyright &copy; 2004-2005  David A. Hall
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

public class BinaryFunctorRef<T1,T2,R> implements FunctorRef<R,BinaryFunctor<T1,T2,R>> {
    private BinaryFunctor<T1,T2,R> _bf;
    private Class<?>[] _argType = new Class[2];
//    private String[] _argName = new String[2]; // unused, for now
    private Class<R> _returnType;
    
    public BinaryFunctorRef (BinaryFunctor<T1,T2,R> bf, Class<?> arg1Type, Class<?> arg2Type,
                             Class<R> returnType)
    {
        _bf = bf;
        _argType[0] = arg1Type;
        _argType[1] = arg2Type;
        _returnType = returnType;        
    }

    
    // FunctorRef implementation
    public BinaryFunctor<T1,T2,R> getFunctor() { return _bf; }
    public Class<R> getReturnType() { return _returnType; }
    public int getNumberArgs() { return _argType.length; }
    public Class<?> getArgType(int i) { return _argType[i]; }
    public String getArgName(int i) { return /*_argName[i];*/ "foo"; }
    public boolean isConstant() { return false; }
    public boolean isIdentity() { return false; }

    // Object implementation
    public String toString() { return _bf.toString()+":"+getReturnType().getName(); }
}
