// ============================================================================
// $Id: EnumHandler.java,v 1.6 2009/06/27 23:57:07 davidahall Exp $
// Copyright (c) 2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.parser;

import java.text.MessageFormat;
import net.sf.jga.fn.adaptor.Constant;

/**
 * Service class that tests if the type is an enum and the name is one of the
 * values of the enumeration.  This class exists and is kept trivially small
 * in order to support Java 1.4 and 1.5 from a single codebase: the build
 * procedure replaces this class with a dummy that does not require a Java 1.5
 * Class.class file be available.
 * <p>
 * Copyright &copy; 2005  David A. Hall
 */

class EnumHandler {
    static final boolean handlesEnums = true;
    
    @SuppressWarnings("unchecked")
    // - We can't wildcard the Class reference in the signature because that causes the compiler 
    // to object since the captured wildcard associated with the class reference isn't known at 
    // compile time to be of the correct type for the Enum.valueOf method.
    // - We can't genericize the Constant for a type given at runtime, or even use an extends
    // wildcard since we're checking the type at runtime.  
    static FunctorRef<?,?> checkEnum(Class type, String name) throws ParseException {
        if (type.isEnum()) {
            try {
                return new GeneratorRef(new Constant(Enum.valueOf(type, name)), type);
            }
            catch (IllegalArgumentException x) {
                String msg = "enum {0} has no {1} value";
                Object[] args = new Object[] { type.getName(), name };
                ParseException px = new ParseException(MessageFormat.format(msg, args));
                px.initCause(x);
                throw px;
            }
        }

        return null;
    }
}
