// ============================================================================
// $Id: FunctorParser.java,v 1.6 2008/02/08 04:22:34 davidahall Exp $
// Copyright (c) 2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.parser;

import net.sf.jga.fn.BinaryFunctor;
import net.sf.jga.fn.Generator;
import net.sf.jga.fn.UnaryFunctor;

/**
 * Parses strings containing JXFG scripts and produces functors.
 * <p>
 * Copyright &copy; 2005  David A. Hall
 */

public interface FunctorParser  {
    /**
     * Parses the string to create a Generator.
     */
    public Generator<?> parseGenerator(String str) throws ParseException;
    
    /**
     * Parses the string to create a UnaryFunctor that takes an argument of the
     * given type.
     */
    public <T1> UnaryFunctor<T1,?> parseUnary(String str, Class<T1> argType) throws ParseException;

    /**
     * Parses the string to create a BinaryFunctor that takes arguments of the
     * given types.
     */
    public <T1,T2> BinaryFunctor<T1,T2,?> parseBinary(String str, Class<T1> arg1Type, Class<T2> arg2Type)
        throws ParseException;
    
    /**
     * Returns the type of object returned by the last functor parsed.
     * @throws IllegalStateException if the parser has not been used or if 
     * parsing the last functor resulted in an exception being thrown.
     */
    public Class <?>getReturnType();
}

