// ============================================================================
// $Id: FunctorRef.java,v 1.8 2009/06/27 23:57:39 davidahall Exp $
// Copyright (c) 2004-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.parser;

import net.sf.jga.fn.Functor;

/**
 * Contains and describes a functor being built by the functor parser.  Many of the
 * parsing methods return and/or accept instances of this interface that describe a
 * functor, describe its broad type, the number, names, and types of its arguments,
 * and the type returned.
 * <p>
 * Copyright &copy; 2004-2005  David A. Hall
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

public interface FunctorRef<R,F extends Functor<R>> {

    /**
     * Returns the functor to which this refers.
     */
    public F getFunctor();

    /**
     * returns the type of objects that the current functor returns.
     */
    public Class<R> getReturnType();

    /**
     * returns the number of arguments that the current functor requires. Currently,
     * the implementations tend to be hard coded.
     */
    public int getNumberArgs();

    /**
     * returns the type of the i'th argument to the current functor.
     * @throws IllegalArgumentException if the current functor takes fewer than i
     * arguments (or if i is < 0)
     */
    public Class<?> getArgType(int i);
    
    /**
     * returns the name of the i'th argument to the current functor, as described
     * in the expression being parsed.
     * @throws IllegalArgumentException if the current functor takes fewer than i
     * arguments (or if i is < 0)
     */
    public String getArgName(int i);
    
    
    /**
     * Returns TRUE if the underlying functor is a constant generator
     */
    public boolean isConstant();
    
    /**
     * Returns TRUE if the underlying functor is a unary identity
     */
    public boolean isIdentity();
}

