// ============================================================================
// $Id: Multiplex.java,v 1.1 2009/06/27 23:53:56 davidahall Exp $
// Copyright (c) 2004-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.parser;

import net.sf.jga.fn.Functor;
import net.sf.jga.fn.UnaryFunctor;

/**
 * Routes an array of values to a given array of unary functions, returning an array 
 * containing the results.  If the number of values passed to evaluation is less than 
 * the number of functions, then nulls are returned in the unused positions.  If the 
 * number of values passed to evaluation is greated than the number of functions, 
 * the excess arguments are ignored.
 */
@SuppressWarnings("unchecked") // use of rawtype UnaryFunctors in an array -- it's a 
// heterogenous collection of functors, so a single generic type is not appropriate.
public class Multiplex extends Functor<Object[]> {
    
    private static final long serialVersionUID = 1060042935668073844L;
    
    // The functions to be evaluated at runtime 
    private UnaryFunctor[] branches;
    
    
    public Multiplex(UnaryFunctor[] branches) {
        this.branches = branches;
    }
    
    /**
     * Returns an array containing the results obtained by passing the args to the 
     * array of functions given at construction.
     * @throws ClassCastException if one of the arguments is not of the correct type 
     * for the corresponding function.
     */
    @Override 
    public Object[] eval(Object... args) {
        Object[] result = new Object[branches.length];
        for (int i = 0; i < branches.length; ++i){
            if (i >= args.length)
                break;
            
            result[i] = branches[i].fn(args[i]);
        }
        
        return result;
    }
}
