// ============================================================================
// $Id: GenericActionListener.java,v 1.5 2005/08/02 23:45:21 davidahall Exp $
// Copyright (c) 2004-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================
package net.sf.jga.swing;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import net.sf.jga.fn.UnaryFunctor;
import net.sf.jga.fn.adaptor.ConstantUnary;
import net.sf.jga.fn.property.ArrayUnary;
import net.sf.jga.fn.property.InvokeMethod;
import net.sf.jga.fn.property.InvokeNoArgMethod;

/**
 * Listener that will execute a given functor on every action event.
 * <p>
 * Copyright &copy; 2004-2005  David A. Hall
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

public class GenericActionListener implements ActionListener {

    /**
     * Builds a listener invokes the given no-arg method every time an
     * action event fires.
     */

    static public <T> GenericActionListener
    buildListener(Class<T> type, T object, String methodName) {
        return new GenericActionListener(
                   new InvokeNoArgMethod<T,Object>(type,methodName)
                        .compose(new ConstantUnary<ActionEvent,T>(object)));
    }

    /**
     * Builds a listener that passes each action event it receives to the
     * given method.
     */

    static public <T> GenericActionListener
    buildEventListener(Class<T> type, T object, String methodName) {
        return new GenericActionListener(
                   new InvokeMethod<T,Object>(type,methodName,ActionEvent.class)
                        .compose(new ConstantUnary<ActionEvent,T>(object),
                                 new ArrayUnary<ActionEvent>()));
    }
    
    // The functor to execute
    private UnaryFunctor<ActionEvent,?> _fn;

    /**
     * Builds an action listener that executes the given functor for every
     * action event it receives.
     */
    public GenericActionListener(UnaryFunctor<ActionEvent,?> fn) {
        _fn = fn;
    }

    // -------------------------------------------------------------------------
    // ActionListener interface
    // -------------------------------------------------------------------------

    /**
     * Passes the action event to the functor given at construction.  The
     * result of the functor, if any, is discareded.
     */
    public void actionPerformed(ActionEvent actionEvent) {
        _fn.fn(actionEvent);
    }
}
