// ============================================================================
// $Id: GenericListCellRenderer.java,v 1.12 2009/01/19 03:37:21 davidahall Exp $
// Copyright (c) 2003-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================
package net.sf.jga.swing;

import java.awt.Component;
import javax.swing.DefaultListCellRenderer;
import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JList;
import net.sf.jga.fn.UnaryFunctor;
import net.sf.jga.fn.string.DefaultFormat;
import net.sf.jga.util.Formattable;

/**
 * ListCellRenderer that passes the contents through a functor to render the
 * contents, instead of calling toString() on the contents.  The functor must
 * take an argument of some type T and return a string.  It is up to the user
 * to ensure that the contents of the list are of the correct type, otherwise,
 * a ClassCastException will be thrown during the component's paint routine.
 * <p>
 * Copyright &copy; 2002-2005  David A. Hall
 *
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

public class GenericListCellRenderer<T> extends DefaultListCellRenderer implements Formattable<T> {

    static final long serialVersionUID = -8781266653605837205L;

    private UnaryFunctor<T,String> _uf;

    public GenericListCellRenderer() {
        this(new DefaultFormat<T>());
    }
    
    public GenericListCellRenderer (UnaryFunctor<T,String> uf) {
        if (uf == null) {
            throw new IllegalArgumentException("Non-null functor required");
        }
        
        _uf = uf;         
    }
    

    @SuppressWarnings("unchecked")
    // There's no way to guarantee that the user has configured this correctly, ie, 
    // ensure that the indexed value in the JList is of type T.
    public Component getListCellRendererComponent(JList list, Object value, int idx,
                                                  boolean sel, boolean focus)
    {
        Component comp = super.getListCellRendererComponent(list, value, idx, sel, focus);
        if (comp instanceof JLabel && ! (value instanceof Icon)) {
            ((JLabel) comp).setText(_uf.fn((T)value));
        }

        return comp;
    }
    
    // ---------------------
    // Formattable interface
    // ---------------------
    
    /**
     * Changes the format of an existing renderer
     */
    public void setFormat(UnaryFunctor<T,String> formatter) {
        if (formatter == null) {
            throw new IllegalArgumentException("Non-null functor required");
        }
        
        _uf = formatter;
    }
}


