// ============================================================================
// $Id: ComponentEditor.java,v 1.2 2005/08/14 20:25:52 davidahall Exp $
// Copyright (c) 2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.swing.spreadsheet;

import java.awt.Component;
import java.awt.KeyboardFocusManager;
import javax.swing.AbstractCellEditor;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.table.TableCellEditor;

/**
 * TableCellEditor used when the contents of the cell are a JComponent.  The
 * component itself will be used as the editor
 * <p>
 * Copyright &copy; 2005  David A. Hall
 */

public class ComponentEditor extends AbstractCellEditor implements TableCellEditor {

    static final long serialVersionUID = -2906356624578134466L;

    // default instance for general use
    static private ComponentEditor _instance;

    /**
     * Returns a sharable ComponentEditor instance.
     */
    static public synchronized ComponentEditor getInstance() {
        if (_instance == null)
            _instance = new ComponentEditor();

        return _instance;
    }

    public ComponentEditor (){
    }

    // Temporarily stores the 'value' of the editor, from the time that the component
    // is to be shown to the user to the time that the user is finished with it, in
    // which case the it is returned by the getCellEditorValue() method during the
    // handling of the editingStopped message.
    private Object _value;

    // Bogus component used when we get an unexpected value
    private JLabel _label = new JLabel();
    
    // ---------------------------------
    // AbstractCellEditor Implementation
    // ---------------------------------

    /**
     * Returns the value of the cell once the editing is complete.  The value is the
     * same component that was stored in the cell, but it's state may have been changed
     * in some way.
     */
    public Object getCellEditorValue() {
        return _value;
    }
        
    // ------------------------------
    // TableCellEditor Implementation
    // ------------------------------

    /**
     * Returns the Component stored in the table at the given cell (expecting that the
     * component is being passed as the 'object' parm), so that it may be temporarily
     * made available to the user for interaction.
     */
    public Component getTableCellEditorComponent(JTable table, Object object, boolean isSelected,
                                                 int row, int col)
    {
        _value = object;
        
        if(object instanceof JComponent) {
            return (JComponent) _value;
        }

        // We don't know how to handle non-component values, so let's just display the value
        // we were givin in something the user can't mess with.  We'll do our best to pretend
        // to be a default renderer

        _label.setForeground(table.getSelectionForeground());
        _label.setBackground(table.getSelectionBackground());
        _label.setBorder(UIManager.getBorder("Table.focusCellHighlightBorder"));
        _label.setFont(table.getFont());
        _label.setText(object.toString());
        _label.setOpaque(true);
        return _label;
    }

    /**
     * Overrides stopCellEditing to remove the focus from the component stored in
     * the currently active cell.
     */
	public boolean stopCellEditing() {
        KeyboardFocusManager.getCurrentKeyboardFocusManager().clearGlobalFocusOwner();
	    return super.stopCellEditing();
	}
}
