// ============================================================================
// $Id: ComponentRenderer.java,v 1.2 2005/08/14 20:27:30 davidahall Exp $
// Copyright (c) 2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.swing.spreadsheet;

import java.awt.Component;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;

/**
 * TableCellRenderer used when the contents of the cell are a JComponent.  The
 * component itself will be used as the rendering component.
 * <p>
 * Copyright &copy; 2005  David A. Hall
 */

public class ComponentRenderer extends DefaultTableCellRenderer implements TableCellRenderer {

    static final long serialVersionUID = -700135148337938417L;

    // default instance for general usage
    static private ComponentRenderer _instance;

    /**
     * Returns a sharable ComponentRenderer instance.
     */
    static public synchronized ComponentRenderer getInstance() {
        if (_instance == null)
            _instance = new ComponentRenderer();

        return _instance;
    }

    public ComponentRenderer () {}

    // --------------------------------
    // TableCellRenderer Implementation
    // --------------------------------

    /**
     * Returns the Component that is stored in the table at the designated cell.
     */
    public Component getTableCellRendererComponent(JTable table, Object object, boolean isSelected,
                                                   boolean hasFocus, int row, int col)
    {
        if(object instanceof JComponent) {
            return (JComponent) object;
        }

        // Rather than throw an exception because we're used improperly,
        // fall back to default table behaviour.
        return super.getTableCellRendererComponent(table, object, isSelected, hasFocus, row, col);
    }
}
