// ============================================================================
// $Id: ChainedComparator.java,v 1.4 2005/08/02 23:45:22 davidahall Exp $
// Copyright (c) 2003-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================
package net.sf.jga.util;

import java.io.Serializable;
import java.util.Comparator;

// UNTESTED

/**
 * Comparator wrapper that uses a pair of comparators internally.  The primary
 * comparator is evaluated first, and if it returns 0 (equal), then this returns
 * the result of the secondary comparator.
 *
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

public class ChainedComparator<T> implements Comparator<T>, Serializable
{
    static final long serialVersionUID = -8015644817201268699L;

    private Comparator<T> _primary;
    private Comparator<T> _secondary;
    
    public ChainedComparator(Comparator<T> primary, Comparator<T> secondary) {
        if (primary == null || secondary == null) {
            String msg = "Two Comparators are required: at least one was null";
            throw new IllegalArgumentException(msg);
        }

        _primary = primary;
        _secondary = secondary;
    }

    public int compare(T x, T y) {
        int result = _primary.compare(x,y);
        return (result == 0) ? _secondary.compare(x,y) : result;
    }
}
