// ============================================================================
// $Id: CollectionUtils.java,v 1.3 2008/12/20 21:07:50 davidahall Exp $
// Copyright (c) 2006  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================
package net.sf.jga.util;

import java.util.Collection;
import java.util.Iterator;

/**
 * General utilites for working with collections.  These are generally extensions
 * to what's already defined in the standard java.util.Collection class
 * <p>
 * Copyright &copy; 2006  David A. Hall
 *
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

public class CollectionUtils {
    /**
     * Adds all of the elements of the iterator to the collection.  If necessary and possible,
     * the collection will be enlarged: if enlarging the collection is not possible, then a
     * runtime exception will be thrown.  This is an augmentation of the
     * Collection.addAll(Collection) API method.
     * @return true if the state of the collection was changed.
     */
    static public <T> boolean addAll(Collection<? super T> cout, Iterator<T> iter) {
        boolean b = false;
        while(iter.hasNext()) {
            b |= cout.add(iter.next());
        }

        return b;
    }    


    /**
     * Adds all of the arguments to the collection.  If necessary and possible, the collection
     * will be enlarged: if enlarging the collection is not possible, then a runtime exception
     * will be thrown.
     * Augmentation of the Collection.addAll(Collection) API method.
     * @return true if the state of the collection was changed.
     */
    static public <T> boolean addAll(Collection<? super T> cout, T... values) {
        boolean b = false;
        for(int i = 0; i < values.length; ++i) {
            b |= cout.add(values[i]);
        }

        return b;
    }
    

    /**
     * Adds all of the elements of the iterable resource to the collection and returns the
     * collection. If necessary and possible, the collection will be enlarged: if enlarging
     * the collection is not possible, then a runtime exception is thrown.
     */
    static public <T,TCollection extends Collection<? super T>> TCollection
    append(TCollection cout, Iterable<T> iterable)
    {
        addAll(cout, iterable.iterator());
        return cout;
    }


    /**
     * Adds all of the elements of the iterator to the collection and returns the collection.
     * If necessary and possible, the collection will be enlarged: if enlarging the collection
     * is not possible, then a runtime exception is thrown.
     */
    static public <T,TCollection extends Collection<? super T>> TCollection
    append(TCollection cout, Iterator<T> iter)
    {
        addAll(cout, iter);
        return cout;
    }


    /**
     * Adds all of the arguments  to the collection and returns the collection.  If necessary
     * and possible, the collection will be enlarged: if enlarging the collection is not
     * possible, then the runtime exception thrown.  A similar method exists in Java 1.5,
     * although it returns boolean rather than the collection.
     */
    static public <T,TCollection extends Collection<? super T>> TCollection
    append(TCollection cout, T... values)
    {
        addAll(cout, values);
        return cout;
    }
}
