// ============================================================================
// $Id: EmptyIterator.java,v 1.6 2005/08/02 23:45:22 davidahall Exp $
// Copyright (c) 2003-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================
package net.sf.jga.util;

import java.util.ListIterator;
import java.util.NoSuchElementException;
import java.util.Iterator;

/**
 * Iterator over an empty set of elements.
 *
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

public class EmptyIterator<T> implements ListIterator<T>, Iterable<T> {
    
    // - - - - - - - - - - -
    // Iterable<T> interface
    // - - - - - - - - - - -

    public Iterator<T> iterator() { return this; }
    
    // - - - - - - - - - - - - -
    // ListIterator<T> interface
    // - - - - - - - - - - - - -
    
    /**
     * Returns false: the set of elements is empty by definition
     * @return false
     */
    public boolean hasNext() { return false; }

    /**
     * throws NoSuchElement exception
     *
     * @return nothing
     */
    public T next() { throw new NoSuchElementException(); }

    /**
     * Returns false: the set of elements is empty by definition
     * @return false
     */
    public boolean hasPrevious() { return false; }

    /**
     * throws NoSuchElement exception
     *
     * @return nothing
     */
    public T previous() { throw new NoSuchElementException(); }

    /**
     * returns the size of the list (0, in this case)
     *
     * @return 0
     */
    public int nextIndex() { return 0; }

    /**
     * returns -1, as there is no previous
     *
     * @return -1;
     */
    public int previousIndex() { return -1; }
    
    /**
     * throws UnsupportedOperationException
     */
    public void remove() { throw new UnsupportedOperationException(); }

    /**
     * throws UnsupportedOperationException
     */
    public void set (T arg) { throw new UnsupportedOperationException(); }

    /**
     * throws UnsupportedOperationException
     */
    public void add (T arg) { throw new UnsupportedOperationException(); }

}
