// ============================================================================
// $Id: EnumerationIterator.java,v 1.7 2005/08/02 23:45:22 davidahall Exp $
// Copyright (c) 2003-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================

package net.sf.jga.util;

import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.Iterator;

/**
 * Adapts an Enumeration to the Iterator and Iterable interfaces.
 * <p>
 * Copyright &copy; 2003-2005  David A. Hall
 *
 * @author <a href="mailto:davidahall@users.sourceforge.net">David A. Hall</a>
 **/

public class EnumerationIterator<T> implements Iterator<T>, Iterable<T> {

    // The base enumeration (as if that wasn't self-evident)
    private Enumeration<T> _base;

    /**
     * Builds an EnumerationIterator that adapts the given enumeration to the
     * Iterator and Iterable interfaces.
     * @throws IllegalArgumentException if the given enumeration is null.
     */
    
    public EnumerationIterator(Enumeration<T> enumeration)
    {
        if (enumeration == null)
            throw new IllegalArgumentException();
        
        _base = enumeration;
    }

    // - - - - - - - - - - -
    // Iterable<T> interface
    // - - - - - - - - - - -

    public Iterator<T> iterator() { return this; }
    
    // - - - - - - - - - - -
    // Iterator<T> interface
    // - - - - - - - - - - -
    
    /**
     * Returns true if the base enumeration has elements remaining.
     * @return true if the base enumeration has elements remaining.
     */

    public boolean hasNext(){
        return _base.hasMoreElements();
    }

    /**
     * Returns the next element in the base enumeration.
     * @return the next element in the base enumeration.
     * @throws NoSuchElementException if the base enumeration is at its end.
     */

    public T next() {
        return _base.nextElement();
    }

    /**
     * throws UnsupportedOperationException: Enumerations do not support the
     * removal of elements.
     * @throws UnsupportedOperationException when called.
     */

    public void remove() throws UnsupportedOperationException {
        throw new UnsupportedOperationException();
    }
}
