// ============================================================================
// $Id: GenericComparator.java,v 1.5 2005/08/02 23:45:22 davidahall Exp $
// Copyright (c) 2003-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================
package net.sf.jga.util;

import net.sf.jga.fn.UnaryFunctor;
import java.io.Serializable;
import java.util.Comparator;

/**
 * Comparator that applies a functor to each argument, then compares the
 * results.  The functor must return Comparable values.
 * <p>
 * Copyright &copy; 2003-2005  David A. Hall
 *
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

public class GenericComparator<T,R extends Comparable<? super R>>
    implements Comparator<T>, Serializable
{

    static final long serialVersionUID = -2163248795946764874L;
    
    // The functor applied to both arguments in order to yield the value to be compared
    private UnaryFunctor<T,R> _fn;

    /**
     * Builds a comparator that uses the given functor to derive the values that are to
     * be compared.
     */
    public GenericComparator(UnaryFunctor<T,R> fn) {
        if (fn == null)
            throw new IllegalArgumentException("Functor is required");

        _fn = fn;
    }

    /**
     * Applies the functor to each of the two arguments, and compares the results.
     */
    public int compare(T x, T y) {
        return _fn.fn(x).compareTo(_fn.fn(y));
    }
}
