// ============================================================================
// $Id: StringTokenizerIterator.java,v 1.7 2007/03/25 03:54:05 davidahall Exp $
// Copyright (c) 2003-2005  David A. Hall
// ============================================================================
// The contents of this file are subject to the Common Development and
// Distribution License (CDDL), Version 1.0 (the License); you may not use this 
// file except in compliance with the License.  You should have received a copy
// of the the License along with this file: if not, a copy of the License is 
// available from Sun Microsystems, Inc.
//
// http://www.sun.com/cddl/cddl.html
//
// From time to time, the license steward (initially Sun Microsystems, Inc.) may
// publish revised and/or new versions of the License.  You may not use,  
// distribute, or otherwise make this file available under subsequent versions 
// of the License.
// 
// Alternatively, the contents of this file may be used under the terms of the 
// GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
// case the provisions of the LGPL are applicable instead of those above. If you 
// wish to allow use of your version of this file only under the terms of the 
// LGPL, and not to allow others to use your version of this file under the 
// terms of the CDDL, indicate your decision by deleting the provisions above 
// and replace them with the notice and other provisions required by the LGPL. 
// If you do not delete the provisions above, a recipient may use your version 
// of this file under the terms of either the CDDL or the LGPL.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// ============================================================================
package net.sf.jga.util;

import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import java.util.Iterator;

/**
 * Adapts a StringTokenizer to the Iterator interface.
 * <p>
 * Copyright &copy; 2003-2005  David A. Hall
 *
 * @author <a href="mailto:davidahall@users.sf.net">David A. Hall</a>
 */

public class StringTokenizerIterator
    implements Iterator<String>, Iterable<String>
{
    private StringTokenizer _base;

    public StringTokenizerIterator(String str) {
        this(new StringTokenizer(str));
    }

    public StringTokenizerIterator(String str, String delim) {
        this(new StringTokenizer(str, delim));
    }
        
    public StringTokenizerIterator(StringTokenizer tok) {
        if (tok == null)
            throw new IllegalArgumentException();
        
        _base = tok;
    }
    
    // - - - - - - - - - - - - - -
    // Iterable<String> interface
    // - - - - - - - - - - - - - -

    public Iterator<String> iterator() { return this; }
    
    // - - - - - - - - - - - - - -
    // Iterator<String> interface
    // - - - - - - - - - - - - - -
    
    /**
     * Returns true if the base tokenizer has tokens remaining.
     */

    public boolean hasNext() {
        return _base.hasMoreTokens();
    }
    
    /**
     * Returns the next string in the base tokenizer.
     * @throws NoSuchElementException if there are no more tokens in the base
     * tokenizer's string
     */

    public String next() {
        return _base.nextToken();
    }

    public void remove() { throw new UnsupportedOperationException(); }
}
